@@ -16,13 +16,14 @@ running Editra.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: Editra.py 53095 2008-04-09 01:39:09Z CJP $"
-__revision__ = "$Revision: 53095 $"
+__svnid__ = "$Id: Editra.py 55000 2008-08-07 00:03:20Z CJP $"
+__revision__ = "$Revision: 55000 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import os
 import sys
+import base64
 import locale
 import time
 import getopt
@@ -55,12 +56,17 @@ import ed_txt
 import ed_event
 import updater
 import plugin
+import ed_ipc
+import ed_msg
 import extern.events as events
 
 #--------------------------------------------------------------------------#
 # Global Variables
 ID_UPDATE_CHECK = wx.NewId()
 
+# Commands (here temporarly)
+APP_CMD_OPEN_WINDOW = u"Editra.OpenWindow"
+
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
 
@@ -82,16 +88,6 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         self._lock = False
         self._windows = dict()
 
-        # Setup Locale
-        locale.setlocale(locale.LC_ALL, '')
-        self.locale = wx.Locale(ed_i18n.GetLangId(profiler.Profile_Get('LANG')))
-        if self.locale.GetCanonicalName() in ed_i18n.GetAvailLocales():
-            self.locale.AddCatalogLookupPathPrefix(ed_glob.CONFIG['LANG_DIR'])
-            self.locale.AddCatalog(ed_glob.PROG_NAME)
-        else:
-            del self.locale
-            self.locale = None
-
         # Setup Plugins after locale as they may have resource that need to
         # be loaded.
         self._pluginmgr = plugin.PluginManager()
@@ -117,9 +113,56 @@ class Editra(wx.App, events.AppEventHandlerMixin):
 
         """
         self.SetAppName(ed_glob.PROG_NAME)
+
         self._log = dev_tool.DEBUGP
         self._log("[app][info] Editra is Initializing")
 
+        self._isfirst = False # Is the first instance
+        self._instance = None
+
+        if ed_glob.SINGLE:
+            # Setup the instance checker
+            instance_name = u"%s-%s" % (self.GetAppName(), wx.GetUserId())
+            self._instance = wx.SingleInstanceChecker(instance_name)
+            if self._instance.IsAnotherRunning():
+                try:
+                    opts, args = getopt.getopt(sys.argv[1:], "dhv",
+                                               ['debug', 'help', 'version'])
+                except getopt.GetoptError, msg:
+                    self._log("[app][err] %s" % str(msg))
+                    args = list()
+
+                if not len(args):
+                    args.append(APP_CMD_OPEN_WINDOW)
+
+                rval = ed_ipc.SendCommands(args, profiler.Profile_Get('SESSION_KEY'))
+            else:
+                self._log("[app][info] Starting Ipc server...")
+                # Set the session key and save it to the users profile so
+                # that other instances can access the server
+                key = unicode(base64.b64encode(os.urandom(8), 'zZ'))
+                key = wx.GetUserName() + key
+                profiler.Profile_Set('SESSION_KEY', key)
+                profiler.Profile_Set('ISBINARY', hasattr(sys, 'frozen'))
+                path = profiler.Profile_Get('MYPROFILE')
+                profiler.Profile().Write(path)
+                self._server = ed_ipc.EdIpcServer(self, profiler.Profile_Get('SESSION_KEY'))
+                self._server.start()
+                self._isfirst = True
+        else:
+            self._isfirst = True
+
+        # Setup Locale
+        locale.setlocale(locale.LC_ALL, '')
+        self.locale = wx.Locale(ed_i18n.GetLangId(profiler.Profile_Get('LANG')))
+        if self.locale.GetCanonicalName() in ed_i18n.GetAvailLocales():
+            self.locale.AddCatalogLookupPathPrefix(ed_glob.CONFIG['LANG_DIR'])
+            self.locale.AddCatalog(ed_glob.PROG_NAME)
+        else:
+            del self.locale
+            self.locale = None
+
+        # Setup the Error Reporter
         if profiler.Profile_Get('REPORTER', 'bool', True):
             sys.excepthook = dev_tool.ExceptionHook
 
@@ -128,9 +171,19 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         self.Bind(wx.EVT_MENU, self.OnNewWindow, id=ed_glob.ID_NEW_WINDOW)
         self.Bind(wx.EVT_MENU, self.OnCloseWindow)
         self.Bind(ed_event.EVT_NOTIFY, self.OnNotify)
+        self.Bind(ed_ipc.EVT_COMMAND_RECV, self.OnCommandRecieved)
 
         return True
 
+    def Destroy(self):
+        """Destroy the application"""
+        try:
+            # Cleanup the instance checker
+            del self._instance
+        except AttributeError:
+            pass
+        wx.App.Destroy(self)
+
     def Exit(self, force=False):
         """Exit the program
         @postcondition: If no toplevel windows are present program will exit.
@@ -140,7 +193,17 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         self._pluginmgr.WritePluginConfig()
         profiler.Profile().Write(profiler.Profile_Get('MYPROFILE'))
         if not self._lock or force:
-            wx.App.Exit(self)
+            if hasattr(self, 'server'):
+                self.server.ShutDown()
+
+            try:
+                # Cleanup the instance checker
+                del self._instance
+            except AttributeError:
+                pass
+
+            # Exit the app
+            wx.App.ExitMainLoop(self)
 
     def GetLocaleObject(self):
         """Get the locale object owned by this app. Use this method to add
@@ -190,6 +253,19 @@ class Editra(wx.App, events.AppEventHandlerMixin):
 
         return awin
 
+    def GetCurrentBuffer(self):
+        """Get the current buffer from the active window or None
+        @return: EditraStc
+
+        """
+        win = self.GetTopWindow()
+        if getattr(win, '__name__', None) != u"MainWindow":
+            win = self.GetActiveWindow()
+            if win is None:
+                return win
+                
+        return win.GetNotebook().GetCurrentCtrl()
+
     def GetMainWindows(self):
         """Returns a list of all open main windows
         @return: list of L{MainWindow} instances of this app (list may be empty)
@@ -238,6 +314,13 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         """
         return self._lock
 
+    def IsOnlyInstance(self):   
+        """Check if this app is the the first instance that is running
+        @return: bool
+
+        """
+        return self._isfirst
+
     def Lock(self):
         """Locks the app from exiting
         @postcondition: program is locked from exiting
@@ -317,6 +400,30 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         else:
             evt.Skip()
 
+    def OnCommandRecieved(self, evt):
+        """Recieve commands from the IPC server
+        @todo: move command processing into own module
+
+        """
+        cmds = evt.GetCommands()
+        for cmdstr in cmds:
+            if u"::" in cmdstr:
+                target, cmd = cmdstr.split(u"::")
+                if target == u"Cmd.EditraStc":
+                    cbuf = self.GetCurrentBuffer()
+                    if cbuf is not None and hasattr(cbuf, cmd):
+                        try:
+                            getattr(cbuf, cmd)()
+                        except:
+                            self._log("[app][err] Invalid Command %s" % cmdstr)
+            else:
+                if cmdstr == APP_CMD_OPEN_WINDOW:
+                    self.OpenNewWindow()
+                elif len(cmdstr):
+                    self.MacOpenFile(cmdstr)
+                else:
+                    self._log("[app][warn] Unknown Command %s" % cmdstr)
+
     def OnCloseWindow(self, evt):
         """Close the currently active window
         @param evt: wx.MenuEvent
@@ -486,6 +593,7 @@ def InitConfig():
             if mode == 'GUI_DEBUG':
                 profiler.Profile_Set('MODE', 'DEBUG')
 
+            profiler.Profile_Del('LASTCHECK')
             #---- End Temporary Profile Adaptions ----#
 
             # Write out updated profile
@@ -529,6 +637,66 @@ def InitConfig():
 
     return profile_updated
 
+#--------------------------------------------------------------------------#
+def PrintHelp():
+    """Print command line help
+    @postcondition: Help is printed and program exits
+
+    """
+    print ("Editra - %s - Developers Text Editor\n"
+       "Cody Precord (2005-2008)\n\n"
+       "usage: Editra [arguments] [files... ]\n\n"
+       "Short Arguments:\n"
+       "  -d         Turn on console debugging\n"
+       "  -D         Turn off console debugging (overrides preferences)\n"
+       "  -h         Show this help message\n"
+       "  -v         Print version number and exit\n"
+       "  -S         Disable single instance checker\n"
+       "\nLong Arguments:\n"
+       "  --debug    Turn on console debugging\n"
+       "  --help     Show this help message\n"
+       "  --auth     Print the ipc server info\n"
+       "  --version  Print version number and exit\n"
+      ) % ed_glob.VERSION
+    os._exit(0)
+
+#--------------------------------------------------------------------------#
+
+def ProcessCommandLine():
+    """Process the command line switches
+    @return: tuple ([switches,], [args,])
+
+    """
+    try:
+        opts, args = getopt.getopt(sys.argv[1:], "dhvDS",
+                                   ['debug', 'help', 'version', 'auth'])
+    except getopt.GetoptError, msg:
+        return list(), list()
+
+    # Process command line options
+    opts = [opt[0] for opt in opts if len(opt) == 2]
+    for opt in list(opts):
+        if opt in ['-h', '--help']:
+            PrintHelp()
+        elif opt in ['-v', '--version']:
+            print "%s - v%s - Developers Editor" % (ed_glob.PROG_NAME, \
+                                                    ed_glob.VERSION)
+            os._exit(0)
+        elif opt in ['-d', '--debug'] and '-D' not in opts:
+            ed_glob.DEBUG = True
+            opts.remove(opt)
+        elif opt == '-D':
+            ed_glob.DEBUG = False
+            opts.remove('-D')
+        elif opt == '-S':
+            ed_glob.SINGLE = False # Disable single instance checker
+            opts.remove(opt)
+        else:
+            pass
+
+    # Return any unprocessed arguments
+    return opts, args
+
 #--------------------------------------------------------------------------#
 
 def Main():
@@ -537,57 +705,47 @@ def Main():
               an instance of Editra and starts the main loop.
 
     """
-    try:
-        opts, args = getopt.getopt(sys.argv[1:], "dhv",
-                                   ['debug', 'help', 'version'])
-    except getopt.GetoptError, msg:
-        dev_tool.DEBUGP("[main][err] %s" % str(msg))
-        opts = list()
-        args = list()
-
-    # Process command line options
-    if len(opts):
-        if True in [x[0] in ['-h', '--help'] for x in opts]:
-            print ("Editra - %s - Developers Text Editor\n"
-                   "Cody Precord (2005-2007)\n\n"
-                   "usage: Editra [arguments] [files... ]\n\n"
-                   "Short Arguments:\n"
-                   "  -d         Turn on console debugging\n"
-                   "  -h         Show this help message\n"
-                   "  -v         Print version number and exit\n"
-                   "\nLong Arguments:\n"
-                   "  --debug    Turn on console debugging\n"
-                   "  --help     Show this help message\n"
-                   "  --version  Print version number and exit\n"
-                  ) % ed_glob.VERSION
-            os._exit(0)
-
-        if True in [x[0] in ['-v', '--version'] for x in opts]:
-            print "%s - v%s - Developers Editor" % (ed_glob.PROG_NAME, \
-                                                    ed_glob.VERSION)
-            os._exit(0)
-
-        if True in [x[0] in ['-d', '--debug'] for x in opts]:
-            ed_glob.DEBUG = True
+    opts, args = ProcessCommandLine()
 
     # We are ready to run so fire up the config and launch the app
     profile_updated = InitConfig()
 
+    # Put extern subpackage on path so that bundled external dependancies
+    # can be found if needed.
+    if not hasattr(sys, 'frozen'):
+        epath = os.path.join(os.path.dirname(__file__), 'extern')
+        if os.path.exists(epath):
+            sys.path.append(epath)
+
     # Create Application
-    dev_tool.DEBUGP("[main][info] Initializing Application...")
+    dev_tool.DEBUGP("[main][app] Initializing application...")
     editra_app = Editra(False)
 
+    # Print ipc server authentication info
+    if '--auth' in opts:
+        opts.remove('--auth')
+        print "port=%d,key=%s" % (ed_ipc.EDPORT,
+                                  profiler.Profile_Get('SESSION_KEY'))
+
+    # Check if this is the only instance, if its not exit since
+    # any of the opening commands have already been passed to the
+    # master instance
+    if not editra_app.IsOnlyInstance():
+        dev_tool.DEBUGP("[main][info] Second instance exiting...")
+        editra_app.Destroy()
+        os._exit(0)
+
     if profile_updated:
         # Make sure window iniliazes to default position
         profiler.Profile_Del('WPOS')
         wx.MessageBox(_("Your profile has been updated to the latest "
                         "version") + u"\n" + \
-                      _("Please check the preferences dialog to reset "
+                      _("Please check the preferences dialog to check "
                         "your preferences"),
                       _("Profile Updated"))
 
     # Splash a warning if version is not a final version
-    if profiler.Profile_Get('APPSPLASH') and int(ed_glob.VERSION[0]) < 1:
+    if profiler.Profile_Get('APPSPLASH'):
         import edimage
         splash_img = edimage.splashwarn.GetBitmap()
         splash = wx.SplashScreen(splash_img, wx.SPLASH_CENTRE_ON_PARENT | \
@@ -620,10 +778,7 @@ def Main():
 
     # Do update check, only check if its been more than a day since the last
     # check
-    tval = sum(time.localtime(time.time())[:3])
-    if profiler.Profile_Get('CHECKUPDATE', default=True) and \
-       tval - profiler.Profile_Get('LASTCHECK', 'int', default=0) > 1:
-        profiler.Profile_Set('LASTCHECK', tval)
+    if profiler.Profile_Get('CHECKUPDATE', default=True):
         uthread = updater.UpdateThread(editra_app, ID_UPDATE_CHECK)
         uthread.start()
 
