@@ -20,8 +20,8 @@ For example usage of these classes see ed_log and the Editra's Launch plugin
 Class OutputBuffer:
 This is the main class exported by ths module. It provides a readonly output
 display buffer that when used with the other classes in this module provides an
-easy way to display continous output from other processes and threads. It 
-provides two methods for subclasses to override if they wish to perform custom 
+easy way to display continous output from other processes and threads. It
+provides two methods for subclasses to override if they wish to perform custom
 handling.
 
   - Override the ApplyStyles method to do any processing and coloring of the
@@ -45,28 +45,40 @@ be overridden in subclasses to perform extra processing.
                    processes exit code as a parameter.
 
 Class ProcessThread:
-Thread class for running subprocesses and posting the output to an 
+Thread class for running subprocesses and posting the output to an
 L{OutputBuffer} via events.
-       
+
+
+Requirements:
+  * wxPython 2.8
+  * Macintosh/Linux/Unix Python 2.4+
+  * Windows Python 2.5+ (ctypes is needed)
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: outbuff.py 52850 2008-03-27 12:51:51Z CJP $"
-__revision__ = "$Revision: 52850 $"
+__svnid__ = "$Id: outbuff.py 54724 2008-07-20 04:58:16Z CJP $"
+__revision__ = "$Revision: 54724 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import os
 import sys
+import time
+import errno
 import signal
-import subprocess
 import threading
+import subprocess
 import wx
 import wx.stc
 
-# Needed for killing processes on windows
-if sys.platform.startswith('win'):
+# Platform specific modules needed for killing processes
+if subprocess.mswindows:
+    import msvcrt
     import ctypes
+else:
+    import select
+    import fcntl
 
 #--------------------------------------------------------------------------#
 # Globals
@@ -80,6 +92,11 @@ OPB_STYLE_WARN    = 2  # Default Red text styling
 OPB_STYLE_ERROR   = 3  # Default Red/Hotspot text styling
 OPB_STYLE_MAX     = 3  # Highest style byte used by outputbuffer
 
+# All Styles
+OPB_ALL_STYLES = (wx.stc.STC_STYLE_DEFAULT, wx.stc.STC_STYLE_CONTROLCHAR,
+                  OPB_STYLE_DEFAULT, OPB_STYLE_ERROR, OPB_STYLE_INFO,
+                  OPB_STYLE_WARN)
+
 #--------------------------------------------------------------------------#
 
 # Event for notifying that the proces has started running
@@ -120,7 +137,7 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
     styleing and filtering output are also available.
 
     """
-    def __init__(self, parent, id=wx.ID_ANY, 
+    def __init__(self, parent, id=wx.ID_ANY,
                  pos=wx.DefaultPosition,
                  size=wx.DefaultSize,
                  style=wx.BORDER_SUNKEN,
@@ -132,6 +149,10 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         self._updating = threading.Condition(self._mutex)
         self._updates = list()
         self._timer = wx.Timer(self)
+        self._colors = dict(defaultb=(255, 255, 255), defaultf=(0, 0, 0),
+                            errorb=(255, 255, 255), errorf=(255, 0, 0),
+                            infob=(255, 255, 255), infof=(0, 0, 255),
+                            warnb=(255, 255, 255), warnf=(255, 0, 0))
 
         # Setup
         self.__ConfigureSTC()
@@ -146,7 +167,7 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
             self._timer.Stop()
 
     def __ConfigureSTC(self):
-        """Setup the stc to behave/appear as we want it to 
+        """Setup the stc to behave/appear as we want it to
         and define all styles used for giving the output context.
         @todo: make more of this configurable
 
@@ -159,7 +180,7 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         self.SetUndoCollection(False) # Don't keep undo history
         self.SetReadOnly(True)
         self.SetCaretWidth(0)
-        
+
         #self.SetEndAtLastLine(False)
         self.SetVisiblePolicy(1, wx.stc.STC_VISIBLE_STRICT)
 
@@ -194,19 +215,20 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
                 fsize = 11
             else:
                 fsize = 10
-            font = wx.Font(fsize, wx.FONTFAMILY_MODERN, 
+
+            font = wx.Font(fsize, wx.FONTFAMILY_MODERN,
                            wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
         style = (font.GetFaceName(), font.GetPointSize(), "#FFFFFF")
         wx.stc.StyledTextCtrl.SetFont(self, font)
-        
+
         # Custom Styles
-        self.StyleSetSpec(OPB_STYLE_DEFAULT, 
+        self.StyleSetSpec(OPB_STYLE_DEFAULT,
                           "face:%s,size:%d,fore:#000000,back:%s" % style)
         self.StyleSetSpec(OPB_STYLE_INFO,
                           "face:%s,size:%d,fore:#0000FF,back:%s" % style)
         self.StyleSetSpec(OPB_STYLE_WARN,
                           "face:%s,size:%d,fore:#FF0000,back:%s" % style)
-        self.StyleSetSpec(OPB_STYLE_ERROR, 
+        self.StyleSetSpec(OPB_STYLE_ERROR,
                           "face:%s,size:%d,fore:#FF0000,back:%s" % style)
         self.StyleSetHotSpot(OPB_STYLE_ERROR, True)
 
@@ -225,7 +247,7 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
     #---- Public Member Functions ----#
     def AppendUpdate(self, value):
         """Buffer output before adding to window. This method can safely be
-        called from non gui threads to add updates to the buffer. 
+        called from non gui threads to add updates to the buffer.
         @param value: update string to append to stack
 
         """
@@ -269,12 +291,68 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         """
         pass
 
+    def GetDefaultBackground(self):
+        """Get the default text style background color
+        @return: wx.Colour
+
+        """
+        return wx.Colour(*self._colors['defaultb'])
+
+    def GetDefaultForeground(self):
+        """Get the default text style foreground color
+        @return: wx.Colour
+
+        """
+        return wx.Colour(*self._colors['defaultf'])
+
+    def GetErrorBackground(self):
+        """Get the error text style background color
+        @return: wx.Colour
+
+        """
+        return wx.Colour(*self._colors['errorb'])
+
+    def GetErrorForeground(self):
+        """Get the error text style foreground color
+        @return: wx.Colour
+
+        """
+        return wx.Colour(*self._colors['errorf'])
+
+    def GetInfoBackground(self):
+        """Get the info text style background color
+        @return: wx.Colour
+
+        """
+        return wx.Colour(*self._colors['infob'])
+
+    def GetInfoForeground(self):
+        """Get the info text style foreground color
+        @return: wx.Colour
+
+        """
+        return wx.Colour(*self._colors['infof'])
+
+    def GetWarningBackground(self):
+        """Get the warning text style background color
+        @return: wx.Colour
+
+        """
+        return wx.Colour(*self._colors['warnb'])
+
+    def GetWarningForeground(self):
+        """Get the warning text style foreground color
+        @return: wx.Colour
+
+        """
+        return wx.Colour(*self._colors['warnf'])
+
     def IsRunning(self):
         """Return whether the buffer is running and ready for output
         @return: bool
 
         """
-        return self._timer.IsRunning()        
+        return self._timer.IsRunning()
 
     def OnTimer(self, evt):
         """Process and display text from the update buffer
@@ -290,12 +368,73 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         else:
             pass
 
+    def SetDefaultColor(self, fore=None, back=None):
+        """Set the colors for the default text style
+        @keyword fore: Foreground Color
+        @keyword back: Background Color
+
+        """
+        if fore is not None:
+            self.StyleSetForeground(wx.stc.STC_STYLE_DEFAULT, fore)
+            self.StyleSetForeground(wx.stc.STC_STYLE_CONTROLCHAR, fore)
+            self.StyleSetForeground(OPB_STYLE_DEFAULT, fore)
+            self._colors['defaultf'] = fore.Get()
+
+        if back is not None:
+            self.StyleSetBackground(wx.stc.STC_STYLE_DEFAULT, back)
+            self.StyleSetBackground(wx.stc.STC_STYLE_CONTROLCHAR, back)
+            self.StyleSetBackground(OPB_STYLE_DEFAULT, back)
+            self._colors['defaultb'] = back.Get()
+
+    def SetErrorColor(self, fore=None, back=None):
+        """Set color for error text
+        @keyword fore: Foreground Color
+        @keyword back: Background Color
+
+        """
+        if fore is not None:
+            self.StyleSetForeground(OPB_STYLE_ERROR, fore)
+            self._colors['errorf'] = fore.Get()
+
+        if back is not None:
+            self.StyleSetBackground(OPB_STYLE_ERROR, back)
+            self._colors['errorb'] = back.Get()
+
+    def SetInfoColor(self, fore=None, back=None):
+        """Set color for info text
+        @keyword fore: Foreground Color
+        @keyword back: Background Color
+
+        """
+        if fore is not None:
+            self.StyleSetForeground(OPB_STYLE_INFO, fore)
+            self._colors['infof'] = fore.Get()
+
+        if back is not None:
+            self.StyleSetBackground(OPB_STYLE_INFO, back)
+            self._colors['infob'] = back.Get()
+
+    def SetWarningColor(self, fore=None, back=None):
+        """Set color for warning text
+        @keyword fore: Foreground Color
+        @keyword back: Background Color
+
+        """
+        if fore is not None:
+            self.StyleSetForeground(OPB_STYLE_WARN, fore)
+            self._colors['warnf'] = fore.Get()
+
+        if back is not None:
+            self.StyleSetBackground(OPB_STYLE_WARN, back)
+            self._colors['warnb'] = back.Get()
+
     def SetFont(self, font):
         """Set the font used by all text in the buffer
         @param font: wxFont
 
         """
-        self.__SetupStyles(font)
+        for style in OPB_ALL_STYLES:
+            self.StyleSetFont(style, font)
 
     def SetText(self, text):
         """Set the text that is shown in the buffer
@@ -366,7 +505,7 @@ class ProcessBufferMixin:
 
     def DoProcessExit(self, code=0):
         """Override this method to do any post processing after the running
-        task has exited. Typically this is a good place to call 
+        task has exited. Typically this is a good place to call
         L{OutputBuffer.Stop} to stop the buffers timer.
         @keyword code: Exit code of program
 
@@ -423,28 +562,68 @@ class ProcessThread(threading.Thread):
 
         # Attributes
         self.abort = False          # Abort Process
+        self._proc = None           # Process handle
         self._parent = parent       # Parent Window/Event Handler
         self._cwd = cwd             # Path at which to run from
         self._cmd = dict(cmd=command, file=fname, args=args)
         self._env = env
 
-    def __DoOneRead(self, proc):
+    def __DoOneRead(self):
         """Read one line of output and post results.
         @param proc: process to read from
         @return: bool (True if more), (False if not)
 
         """
-        try:
-            result = proc.stdout.readline()
-        except (IOError, OSError):
-            return False
+        if subprocess.mswindows:
+            # Windows nonblocking pipe read implementation
+            read = u''
+            try:
+                handle = msvcrt.get_osfhandle(self._proc.stdout.fileno())
+                avail = ctypes.c_long()
+                ctypes.windll.kernel32.PeekNamedPipe(handle, None, 0, 0,
+                                                     ctypes.byref(avail), None)
+                if avail.value > 0:
+                    read = self._proc.stdout.readline()
+                    if read.endswith(os.linesep):
+                        read = read[:-1 * len(os.linesep)] + "\n"
+                else:
+                    if self._proc.poll() is None:
+                        time.sleep(1)
+                        return True
+                    else:
+                        # Process has Exited
+                        return False
+            except ValueError:
+                return False
+            except (subprocess.pywintypes.error, Exception), msg:
+                if msg[0] in (109, errno.ESHUTDOWN):
+                    return False
 
-        if result == "" or result == None:
-            return False
+        else:
+            # OSX and Unix nonblocking pipe read implementation
+            if self._proc.stdout is None:
+                return False
+
+            flags = fcntl.fcntl(self._proc.stdout, fcntl.F_GETFL)
+            if not self._proc.stdout.closed:
+                fcntl.fcntl(self._proc.stdout,
+                            fcntl.F_SETFL,
+                            flags| os.O_NONBLOCK)
+
+            try:
+                if not select.select([self._proc.stdout], [], [], 1)[0]:
+                    return True
+
+                read = self._proc.stdout.readline()
+                if read == '':
+                    return False
+            finally:
+                if not self._proc.stdout.closed:
+                    fcntl.fcntl(self._proc.stdout, fcntl.F_SETFL, flags)
 
         # Ignore encoding errors and return an empty line instead
         try:
-            result = result.decode(sys.getfilesystemencoding())
+            result = read.decode(sys.getfilesystemencoding())
         except UnicodeDecodeError:
             result = os.linesep
 
@@ -463,15 +642,16 @@ class ProcessThread(threading.Thread):
             return
 
         if wx.Platform != '__WXMSW__':
-            # Try to be 'nice' the first time if that fails be more forcefull
             try:
-                os.kill(pid, signal.SIGABRT)
-            except OSError:
+                self._proc.stdout.close()
+                self._proc.stdout = None
                 os.kill(pid, signal.SIGKILL)
+            except OSError:
+                return
 
             os.waitpid(pid, os.WNOHANG)
         else:
-            # 1 == Terminate
+            # 1 == PROCESS_TERMINATE
             handle = ctypes.windll.kernel32.OpenProcess(1, False, pid)
             ctypes.windll.kernel32.TerminateProcess(handle, -1)
             ctypes.windll.kernel32.CloseHandle(handle)
@@ -491,10 +671,11 @@ class ProcessThread(threading.Thread):
         command = u' '.join([item.strip() for item in [self._cmd['cmd'],
                                                        self._cmd['file'],
                                                        self._cmd['args']]])
-        
-        proc = subprocess.Popen(command.strip(), stdout=subprocess.PIPE,
-                                stderr=subprocess.STDOUT, shell=True,
-                                cwd=self._cwd, env=self._env)
+
+        use_shell = not subprocess.mswindows
+        self._proc = subprocess.Popen(command.strip(), stdout=subprocess.PIPE,
+                                      stderr=subprocess.STDOUT, shell=use_shell,
+                                      cwd=self._cwd, env=self._env)
 
         evt = OutputBufferEvent(edEVT_PROCESS_START,
                                 self._parent.GetId(),
@@ -504,23 +685,24 @@ class ProcessThread(threading.Thread):
         # Read from stdout while there is output from process
         while True:
             if self.abort:
-                self.__KillPid(proc.pid)
-                self.__DoOneRead(proc)
+                self.__KillPid(self._proc.pid)
+                self.__DoOneRead()
+                more = False
                 break
             else:
                 more = False
                 try:
-                    more = self.__DoOneRead(proc)
+                    more = self.__DoOneRead()
                 except wx.PyDeadObjectError:
                     # Our parent window is dead so kill process and return
-                    self.__KillPid(proc.pid)
+                    self.__KillPid(self._proc.pid)
                     return
 
                 if not more:
                     break
 
         try:
-            result = proc.wait()
+            result = self._proc.wait()
         except OSError:
             result = -1
 
