@@ -18,8 +18,8 @@ a gradient using system defined colors.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_cmdbar.py 53692 2008-05-21 22:01:00Z CJP $"
-__revision__ = "$Revision: 53692 $"
+__svnid__ = "$Id: ed_cmdbar.py 54630 2008-07-14 23:38:58Z CJP $"
+__revision__ = "$Revision: 54630 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -332,6 +332,7 @@ class CommandBar(wx.Panel):
             if id_ == ID_SEARCH_CTRL:
                 for kid in self._sizers['search'].GetChildren():
                     kid.Show(True)
+                self.FindWindowById(ID_SEARCH_CTRL).AutoSetQuery()
                 self._sizers['search'].Layout()
             elif id_ == ID_LINE_CTRL:
                 for kid in self._sizers['goto'].GetChildren():
@@ -417,20 +418,27 @@ class CommandExecuter(wx.SearchCtrl):
 
         self.Bind(wx.EVT_TEXT_ENTER, self.OnEnter)
         self.Bind(ed_event.EVT_NOTIFY, self.OnPopupNotify)
+        ed_msg.Subscribe(self._UpdateCwd, ed_msg.EDMSG_UI_NB_CHANGED)
+        ed_msg.Subscribe(self._UpdateCwd, ed_msg.EDMSG_FILE_SAVED)
+
+    def __del__(self):
+        ed_msg.Unsubscribe(self._UpdateCwd)
 
     def _AdjustSize(self):
         """Checks width of text as its added and dynamically resizes
         the control as needed.
-        @postcondition: control is resized to fit the text in it upto a max
-                        width of 75 percent of the client space available.
 
         """
         ext = self.GetTextExtent(self.GetValue())[0]
         curr_w, curr_h = self.GetClientSizeTuple()
-        if ext > curr_w * .85:
-            max_w = self.GetParent().GetClientSize().GetWidth() * .75
-            nwidth = min(ext * 1.18, max_w)
+        if ext > curr_w * .5:
+            max_w = self.GetParent().GetClientSize().GetWidth() * .8
+            nwidth = min(ext * 1.3, max_w)
+            pwidth = self._popup.GetBestSize()[0]
+            if pwidth > nwidth:
+                nwidth = pwidth
             self.SetClientSize((nwidth, curr_h))
+            self._popup.SetSize((nwidth, -1))
         elif ((curr_w > ext * 1.18) and curr_w > 150):
             nwidth = max(ext * 1.18, 150)
             self.SetClientSize((nwidth, curr_h))
@@ -447,6 +455,20 @@ class CommandExecuter(wx.SearchCtrl):
             self.AppendText(val.replace(cval[-1], '', 1))
         else:
             self.SetValue(" ".join([cval[0], val]))
+        self.SetInsertionPoint(self.GetLastPosition())
+
+    def _UpdateCwd(self, msg=None):
+        """Update the current working directory to that of the current
+        buffer.
+
+        """
+        # Only Update if we are the currently active window
+        tlp = self.GetTopLevelParent()
+        if tlp.IsActive():
+            ctrl = tlp.GetNotebook().GetCurrentCtrl()
+            fname = ctrl.GetFileName()
+            if len(fname):
+                self._curdir = os.path.dirname(fname)
 
     def ChangeDir(self, cmd):
         """Change to a directory based on cd command
@@ -679,12 +701,13 @@ class CommandExecuter(wx.SearchCtrl):
         if len(paths):
             pos = self.GetScreenPosition().Get()
             extent = self.GetTextExtent(cstr)
-            self._popup.SetPosition((pos[0] + extent[0], pos[1] + extent[1]))
+            self._popup.SetPosition((pos[0] + extent[0], pos[1] + extent[1] + 7))
             self._popup.SetBestSelection(cmd)
             if not self._popup.IsShown():
                 self._popup.Show()
         else:
             self._popup.Hide()
+        self.SetInsertionPoint(self.GetLastPosition())
 
     def OnEnter(self, evt):
         """Get the currently entered command string and execute it.
@@ -875,7 +898,7 @@ class LineCtrl(wx.SearchCtrl):
 #-----------------------------------------------------------------------------#
 # TODO: merge the common parts of these two classes into a single base class
 
-class PopupList(wx.Frame):
+class PopupList(wx.MiniFrame):
     def __init__(self, parent, choices=list(), pos=wx.DefaultPosition):
 
         style = wx.FRAME_NO_TASKBAR | wx.FRAME_FLOAT_ON_PARENT
@@ -884,7 +907,7 @@ class PopupList(wx.Frame):
         else:
             style = style | wx.SIMPLE_BORDER
 
-        wx.Frame.__init__(self, parent, pos=pos, style=style)
+        wx.MiniFrame.__init__(self, parent, pos=pos, style=style)
 
         # Attributes
         self._list = wx.ListBox(self, choices=choices,
@@ -892,6 +915,8 @@ class PopupList(wx.Frame):
                                       wx.LC_NO_HEADER | wx.NO_BORDER)
 
         # Layout
+        self._list.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
+        self.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(self._list, 1, wx.EXPAND)
         self.SetSizer(sizer)
@@ -900,6 +925,7 @@ class PopupList(wx.Frame):
         self.SetAutoLayout(True)
 
         # Event Handlers
+        self.Bind(wx.EVT_CHAR, lambda evt: parent.GetEventHandler().ProcessEvent(evt))
         self.Bind(wx.EVT_SET_FOCUS, self.OnFocus)
         self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnSelection)
         self.Bind(wx.EVT_SIZE, self.OnSize)
@@ -1073,13 +1099,7 @@ class PopupWinList(wx.PopupWindow):
         self.SetAutoLayout(True)
 
         # Event Handlers
-#         self.Bind(wx.EVT_SET_FOCUS, self.OnFocus)
-#         self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnSelection)
-#         self.Bind(wx.EVT_SIZE, self.OnSize)
-#         self._list.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
-
-#         self._list.SetFocus()
-#         self.Hide()
+        self.Bind(wx.EVT_SIZE, self.OnSize)
 
     def AdvanceSelection(self, down=True):
         """Advance the selection in the list
@@ -1105,6 +1125,12 @@ class PopupWinList(wx.PopupWindow):
         """
         return self._list.GetStringSelection()
 
+    def OnSize(self, evt):
+        csz = self.GetClientSize()
+        csz.SetWidth(csz.x + wx.SystemSettings.GetMetric(wx.SYS_VSCROLL_X))
+        self._list.SetSize(csz)
+        evt.Skip()
+
     def SetBestSelection(self, prefix):
         """Set the selection to the one that bests matches the
         given string.
