@@ -14,8 +14,8 @@ access point for all common variables in the project.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_glob.py 53301 2008-04-22 03:37:58Z CJP $"
-__revision__ = "$Revision: 53301 $"
+__svnid__ = "$Id: ed_glob.py 55000 2008-08-07 00:03:20Z CJP $"
+__revision__ = "$Revision: 55000 $"
 
 __all__ = [ 'CONFIG', 'SB_INFO', 'VERSION', 'PROG_NAME', 'ID_NEW', 'ID_OPEN',
             'ID_CLOSE', 'ID_CLOSEALL', 'ID_SAVE', 'ID_SAVEAS', 'ID_SAVEALL',
@@ -38,7 +38,8 @@ __all__ = [ 'CONFIG', 'SB_INFO', 'VERSION', 'PROG_NAME', 'ID_NEW', 'ID_OPEN',
             'ID_MACRO_PLAY', 'ID_ABOUT', 'ID_HOMEPAGE', 'ID_CONTACT',
             'ID_COMMAND_BAR', 'ID_DOCUMENTATION', 'ID_COMMAND',
             'ID_CLOSE_WINDOW', 'ID_USE_SOFTTABS', 'ID_DUP_LINE', 'ID_TRANSLATE',
-            'I18N_PAGE', 'ID_GOTO_MBRACE', 'ID_HLCARET_LINE'
+            'I18N_PAGE', 'ID_GOTO_MBRACE', 'ID_HLCARET_LINE', 'ID_SHOW_SB',
+            'ID_REVERT_FILE'
 ]
 
 #---- Project Info ----#
@@ -59,6 +60,11 @@ if wx.VERSION < (2, 8, 6, 0, ''):
     wx.MenuItem.GetItemLabel = wx.MenuItem.GetText
     wx.MenuItem.GetItemLabelText = wx.MenuItem.GetLabel
 
+# HACK temporary bandaid to allow the the program to run until a replacement
+#      for this method can be found.
+if wx.VERSION >= (2, 9, 0, 0, ''):
+    wx.Brush.MacSetTheme = lambda x, y: x
+
 #---- Configuration Locations ----#
 # Values set when main loads
 CONFIG = {
@@ -74,6 +80,7 @@ CONFIG = {
 }
 
 DEBUG = False
+SINGLE = True
 
 #---- Object ID's ----#
 # File Menu IDs
@@ -87,6 +94,7 @@ ID_CLOSE_WINDOW  = wx.NewId()
 ID_SAVE          = wx.ID_SAVE
 ID_SAVEAS        = wx.ID_SAVEAS
 ID_SAVEALL       = wx.NewId()
+ID_REVERT_FILE   = wx.ID_REVERT_TO_SAVED
 ID_SAVE_PROFILE  = wx.NewId()
 ID_LOAD_PROFILE  = wx.NewId()
 ID_PRINT         = wx.ID_PRINT
@@ -141,6 +149,7 @@ ID_PREF_WPOS     = wx.NewId()
 ID_PREF_ICON     = wx.NewId()
 ID_PREF_ICONSZ   = wx.NewId()
 ID_PREF_MODE     = wx.NewId()
+ID_PREF_TABICON  = wx.NewId()
 ID_PRINT_MODE    = wx.NewId()
 ID_TRANSPARENCY  = wx.NewId()
 ID_PREF_SPOS     = wx.NewId()
@@ -159,6 +168,7 @@ ID_SHOW_WS       = wx.NewId()
 ID_SHOW_SHELF    = wx.NewId()
 ID_PERSPECTIVES  = wx.NewId()
 ID_INDENT_GUIDES = wx.NewId()
+ID_SHOW_SB       = wx.NewId()
 ID_VIEW_TOOL     = wx.NewId()
 ID_SHELF         = wx.NewId()
 ID_GOTO_LINE     = wx.NewId()
@@ -207,6 +217,7 @@ ID_GENERATOR     = wx.NewId()
 ID_HTML_GEN      = wx.NewId()
 ID_TEX_GEN       = wx.NewId()
 ID_RTF_GEN       = wx.NewId()
+ID_RUN_LAUNCH    = wx.NewId()
 
 # Help Menu IDs
 ID_ABOUT         = wx.ID_ABOUT
@@ -233,6 +244,7 @@ ID_FOLDER            = wx.NewId()
 ID_HARDDISK          = wx.NewId()
 ID_KEY_PROFILES      = wx.NewId()
 ID_PACKAGE           = wx.NewId()
+ID_REFRESH           = wx.ID_REFRESH
 ID_REPORTER          = wx.NewId()
 ID_STOP              = wx.ID_STOP
 ID_THEME             = wx.NewId()
@@ -272,6 +284,7 @@ ID_2_PROF = {
              ID_PREF_SYNTHEME     : 'SYNTHEME',
              ID_PREF_TABS         : 'USETABS',
              ID_PREF_UNINDENT     : 'BSUNINDENT',
+             ID_PREF_TABICON      : 'TABICONS',
              ID_PREF_TABW         : 'TABWIDTH',
              ID_PREF_INDENTW      : 'INDENTWIDTH',
              ID_SESSION           : 'SAVE_SESSION',
@@ -290,7 +303,9 @@ ID_2_PROF = {
              ID_PREF_MODE         : 'MODE',
              ID_PRINT_MODE        : 'PRINT_MODE',
              ID_NEW_WINDOW        : 'OPEN_NW',
-             ID_VI_MODE           : 'VI_EMU'
+             ID_SHOW_SB           : 'STATBAR',
+             ID_VI_MODE           : 'VI_EMU',
+             ID_VIEW_TOOL         : 'TOOLBAR'
 }
 
 # Default Plugins
