@@ -11,13 +11,13 @@ Editra LogViewer
 
 This module provides classes for managing the log display and filtering of its
 messages. The module also exports an implementation of a shelf plugin for
-displaying a LogViewer in the Shelf.
+displaying a L{LogViewer} in the Shelf.
 
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_log.py 52189 2008-02-29 08:01:50Z CJP $"
-__revision__ = "$Revision: 52189 $"
+__svnid__ = "$Id: ed_log.py 54578 2008-07-11 13:37:52Z CJP $"
+__revision__ = "$Revision: 54578 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -104,6 +104,12 @@ class LogViewer(ctrlbox.ControlBox):
         self.Bind(wx.EVT_BUTTON,
                   lambda evt: self._buffer.Clear(), id=wx.ID_CLEAR)
         self.Bind(wx.EVT_CHOICE, self.OnChoice, self._srcfilter)
+        ed_msg.Subscribe(self.OnThemeChange, ed_msg.EDMSG_THEME_CHANGED)
+
+    def __del__(self):
+        """Cleanup and unsubscribe from messages"""
+        ed_msg.Unsubscribe(self.OnThemeChange)
+        super(LogViewer).__del__()
 
     def __DoLayout(self):
         """Layout the log viewer window"""
@@ -114,7 +120,8 @@ class LogViewer(ctrlbox.ControlBox):
 
         # View Choice
         self._srcfilter = wx.Choice(ctrlbar, wx.ID_ANY, choices=[])
-        ctrlbar.AddControl(wx.StaticText(ctrlbar, label=_("Filter") + ":"))
+        ctrlbar.AddControl(wx.StaticText(ctrlbar,
+                                         label=_("Show output from") + ":"))
         ctrlbar.AddControl(self._srcfilter)
 
         # Clear Button
@@ -135,6 +142,16 @@ class LogViewer(ctrlbox.ControlBox):
         """
         self._buffer.SetFilter(self._srcfilter.GetStringSelection())
 
+    def OnThemeChange(self, msg):
+        """Update the buttons icon when the icon theme changes
+        @param msg: Message Object
+
+        """
+        clear = self.FindWindowById(wx.ID_CLEAR)
+        cbmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_DELETE), wx.ART_MENU)
+        clear.SetBitmap(cbmp)
+        clear.Refresh()
+
     def SetSources(self, srclist):
         """Set the list of available log sources in the choice control
         @param srclist: list of log sources
