@@ -15,8 +15,8 @@ main Ui component of the editor that contains all the other components.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_main.py 54034 2008-06-08 22:38:08Z CJP $"
-__revision__ = "$Revision: 54034 $"
+__svnid__ = "$Id: ed_main.py 54634 2008-07-15 02:19:44Z CJP $"
+__revision__ = "$Revision: 54634 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -147,6 +147,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                        (ID_SAVE, self.OnSave),
                                        (ID_SAVEAS, self.OnSaveAs),
                                        (ID_SAVEALL, self.OnSave),
+                                       (ID_REVERT_FILE, self.DispatchToControl),
                                        (ID_SAVE_PROFILE, self.OnSaveProfile),
                                        (ID_LOAD_PROFILE, self.OnLoadProfile),
                                        (ID_EXIT, wx.GetApp().OnExit),
@@ -193,7 +194,9 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                   id=wx.ID_FILE1, id2=wx.ID_FILE9)
 
         # Update UI Handlers
-        self._handlers['ui'].extend([# Edit Menu
+        self._handlers['ui'].extend([# File Menu
+                                     (ID_REVERT_FILE, self.OnUpdateFileUI),
+                                     # Edit Menu
                                      (ID_COPY, self.OnUpdateClipboardUI),
                                      (ID_CUT, self.OnUpdateClipboardUI),
                                      (ID_PASTE, self.OnUpdateClipboardUI),
@@ -389,7 +392,8 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             dlg.Destroy()
         else:
             self.LOG("[ed_main][info] CMD Open File: %s" % fname)
-            self.nb.OpenPage(util.GetPathName(fname), util.GetFileName(fname))
+            self.nb.OpenPage(util.GetPathName(fname),
+                             util.GetFileName(fname), quiet=True)
 
     def GetFrameManager(self):
         """Returns the manager for this frame
@@ -696,6 +700,8 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             result = self.nb.ClosePage()
             if result == wx.ID_CANCEL:
                 self._exiting = False
+                ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGED,
+                                   (self.nb, self.nb.GetSelection()))
                 return True
             controls -= 1
 
@@ -926,7 +932,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                          ID_TRIM_WS, ID_SHOW_EDGE, ID_MACRO_START,
                          ID_MACRO_STOP, ID_MACRO_PLAY, ID_TO_LOWER,
                          ID_TO_UPPER, ID_KWHELPER, ID_USE_SOFTTABS,
-                         ID_GOTO_MBRACE, ID_HLCARET_LINE
+                         ID_GOTO_MBRACE, ID_HLCARET_LINE, ID_REVERT_FILE
                          ])
         menu_ids.extend(active_only)
 
@@ -937,6 +943,24 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         return
 
     # Menu Update Handlers
+    def OnUpdateFileUI(self, evt):
+        """Update filemenu items
+        @param evt: EVT_UPDATE_UI
+
+        """
+        if not self.IsActive():
+            return
+
+        e_id = evt.GetId()
+        evt.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
+        # Slow the update interval to reduce overhead
+        evt.SetUpdateInterval(160)
+        if e_id == ID_REVERT_FILE:
+            ctrl = self.nb.GetCurrentCtrl()
+            evt.Enable(ctrl.GetModify())
+        else:
+            evt.Skip()
+
     def OnUpdateClipboardUI(self, evt):
         """Update clipboard related menu/toolbar items
         @param evt: EVT_UPDATE_UI
