@@ -2,7 +2,7 @@
 # Name: ed_menu.py                                                            #
 # Purpose: Editra's Menubar and Menu related classes                          #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# Copyright: (c) 2007-2008 Cody Precord <staff@editra.org>                    #
 # License: wxWindows License                                                  #
 ###############################################################################
 
@@ -14,8 +14,8 @@ for managing keybindings and profiles is also provided by this module.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_menu.py 53301 2008-04-22 03:37:58Z CJP $"
-__revision__ = "$Revision: 53301 $"
+__svnid__ = "$Id: ed_menu.py 55013 2008-08-07 19:37:52Z CJP $"
+__revision__ = "$Revision: 55013 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -460,10 +460,10 @@ class EdMenuBar(wx.MenuBar):
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_OPEN),
                         _("Open"))
         ## Setup File History in the File Menu
-        filemenu.AppendMenu(ed_glob.ID_FHIST, _("Open Recent"),
+        filemenu.AppendMenu(ed_glob.ID_FHIST, _("Open &Recent"),
                             filehist, _("Recently Opened Files"))
         filemenu.AppendSeparator()
-        filemenu.Append(ed_glob.ID_CLOSE, _("Close Page") + \
+        filemenu.Append(ed_glob.ID_CLOSE, _("&Close Page") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_CLOSE),
                         _("Close Current Page"))
         filemenu.Append(ed_glob.ID_CLOSE_WINDOW,
@@ -484,6 +484,10 @@ class EdMenuBar(wx.MenuBar):
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_SAVEALL),
                         _("Save all open pages"))
         filemenu.AppendSeparator()
+        filemenu.Append(ed_glob.ID_REVERT_FILE, _("Revert to Saved") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_REVERT_FILE),
+                        _("Revert file to last save point"))
+        filemenu.AppendSeparator()
         pmenu = EdMenu()
         pmenu.Append(ed_glob.ID_SAVE_PROFILE, _("Save Profile") + \
                      EdMenuBar.keybinder.GetBinding(ed_glob.ID_SAVE_PROFILE),
@@ -642,6 +646,7 @@ class EdMenuBar(wx.MenuBar):
         viewmenu.Append(ed_glob.ID_GOTO_MBRACE, _("Goto Matching Brace") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_GOTO_MBRACE),
                         _("Move caret matching brace"))
+        viewmenu.AppendSeparator()
         viewmenu.Append(ed_glob.ID_NEXT_MARK, _("Next Bookmark") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_NEXT_MARK),
                         _("View Line of Next Bookmark"))
@@ -649,6 +654,9 @@ class EdMenuBar(wx.MenuBar):
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_PRE_MARK),
                         _("View Line of Previous Bookmark"))
         viewmenu.AppendSeparator()
+        viewmenu.Append(ed_glob.ID_SHOW_SB, ("Status &Bar") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_SHOW_SB),
+                        _("Show Status Bar"), wx.ITEM_CHECK)
         viewmenu.Append(ed_glob.ID_VIEW_TOOL, _("&Toolbar") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_VIEW_TOOL),
                         _("Show Toolbar"), wx.ITEM_CHECK)
@@ -957,6 +965,7 @@ _DEFAULT_BINDING = { # File Menu
                      ed_glob.ID_GOTO_MBRACE : ("Ctrl", "Shift", "B"),
                      ed_glob.ID_NEXT_MARK : ("Alt", "Right"), # Win/Linux
                      ed_glob.ID_PRE_MARK : ("Alt", "Left"), # Win/Linux
+                     ed_glob.ID_SHOW_SHELF : ("Ctrl", "Alt", "S"),
 
                      # Format Menu
                      ed_glob.ID_COMMENT : ("Ctrl", "1"),
@@ -967,7 +976,8 @@ _DEFAULT_BINDING = { # File Menu
 
                      # Tools Menu
                      ed_glob.ID_COMMAND : ("Ctrl", "E"),
-                     ed_glob.ID_KWHELPER : ("Ctrl", "K")
+                     ed_glob.ID_KWHELPER : ("Ctrl", "K"),
+                     ed_glob.ID_RUN_LAUNCH : ("F5",)
                      }
 
 # Set some platform specific keybindigs
@@ -1052,7 +1062,7 @@ def WalkMenu(menu, label, collection):
         if item.IsSubMenu():
             # Ignore dynamically generated menus
             if i_id not in (ed_glob.ID_FHIST, ed_glob.ID_LEXER,
-                            ed_glob.ID_SHELF, ed_glob.ID_PERSPECTIVES):
+                             ed_glob.ID_PERSPECTIVES):
                 ilbl = item.GetItemLabelText()
                 collection[ilbl] = [i_id, ]
                 WalkMenu(item.GetSubMenu(), ilbl, collection)
