@@ -20,8 +20,8 @@ in the notebook see ed_stc.py
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_pages.py 54023 2008-06-08 01:14:19Z CJP $"
-__revision__ = "$Revision: 54023 $"
+__svnid__ = "$Id: ed_pages.py 54780 2008-07-24 02:56:08Z CJP $"
+__revision__ = "$Revision: 54780 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -100,6 +100,7 @@ class EdPages(FNB.FlatNotebook):
         self._pages.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
         self.Bind(wx.EVT_IDLE, self.OnIdle)
         ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
+        ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_NOTEBOOK)
 
         # Add a blank page
         self.NewPage()
@@ -114,6 +115,7 @@ class EdPages(FNB.FlatNotebook):
         to reopen the file again the function will return True else it will
         return False.
         @param path: file to check for
+        @return: bool
 
         """
         result = wx.ID_YES
@@ -127,11 +129,7 @@ class EdPages(FNB.FlatNotebook):
             result = mdlg.ShowModal()
             mdlg.Destroy()
             if result == wx.ID_NO:
-                for page in xrange(self.GetPageCount()):
-                    ctrl = self.GetPage(page)
-                    if path == ctrl.GetFileName():
-                        self.SetSelection(page)
-                        break
+                self.GotoPage(path)
         elif os.path.exists(path) and not os.path.isfile(path):
             result = wx.ID_NO
         else:
@@ -208,16 +206,25 @@ class EdPages(FNB.FlatNotebook):
         """
         self.UpdateAllImages()
 
-    def OpenPage(self, path, filename):
+    def OpenPage(self, path, filename, quiet=False):
         """Open a File Inside of a New Page
         @param path: files base path
         @param filename: name of file to open
+        @keyword quiet: Open/Switch to the file quietly if
+                        it is already open.
 
         """
         path2file = os.path.join(path, filename)
 
         # Check if file needs to be opened
-        if not self._NeedOpen(path2file):
+        # TODO: these steps could be combined together with some
+        #       refactoring of the _NeedOpen method. Requires extra
+        #       testing though to check for dependancies on current
+        #       behavior.
+        if quiet and self.HasFileOpen(path2file):
+            self.GotoPage(path2file)
+            return
+        elif not self._NeedOpen(path2file):
             return
 
         # Create new control to place text on if necessary
@@ -298,18 +305,8 @@ class EdPages(FNB.FlatNotebook):
             control.Show()
             self.control = control
 
-        # Pass directory and file name info to control object to save reference
-        self.control.SetModTime(util.GetFileModTime(path2file))
-        self.frame.AddFileToHistory(path2file)
-        if new_pg:
-            self.AddPage(self.control, filename)
-
-        self.frame.SetTitle("%s - file://%s" % (filename,
-                                                self.control.GetFileName()))
-        self.SetPageText(self.GetSelection(), filename)
-        self.LOG("[ed_pages][evt] Opened Page: %s" % filename)
-
         # Setup Document
+        self.control.SetModTime(util.GetFileModTime(path2file))
         self.control.FindLexer()
         self.control.CheckEOL()
         self.control.EmptyUndoBuffer()
@@ -317,6 +314,16 @@ class EdPages(FNB.FlatNotebook):
         if Profile_Get('SAVE_POS'):
             self.control.GotoPos(self.DocMgr.GetPos(self.control.GetFileName()))
 
+        # Add the buffer to the notebook
+        if new_pg:
+            self.AddPage(self.control, filename)
+
+        self.frame.SetTitle("%s - file://%s" % (filename,
+                                                self.control.GetFileName()))
+        self.frame.AddFileToHistory(path2file)
+        self.SetPageText(self.GetSelection(), filename)
+        self.LOG("[ed_pages][evt] Opened Page: %s" % filename)
+
         # Set tab image
         self.SetPageImage(self.GetSelection(), str(self.control.GetLangId()))
 
@@ -341,6 +348,17 @@ class EdPages(FNB.FlatNotebook):
         self.control = control
         return current_page
 
+    def GotoPage(self, fname):
+        """Go to the page containing the buffer with the given file.
+        @param fname: file path (string)
+
+        """
+        for page in xrange(self.GetPageCount()):
+            ctrl = self.GetPage(page)
+            if fname == ctrl.GetFileName():
+                self.ChangePage(page)
+                break
+
     def GetPageText(self, pg_num):
         """Gets the tab text from the given page number, stripping
         the * mark if there is one.
@@ -366,7 +384,9 @@ class EdPages(FNB.FlatNotebook):
                  notebook.
 
         """
-        return [self.GetPage(page) for page in xrange(self.GetPageCount())]
+        pages = [self.GetPage(page) for page in xrange(self.GetPageCount())]
+        return [page for page in pages 
+                if getattr(page, '__name__', '') == "EditraTextCtrl"]
 
     def HasFileOpen(self, fpath):
         """Checks if one of the currently active buffers has
@@ -491,19 +511,26 @@ class EdPages(FNB.FlatNotebook):
                   "%d to Page: %d\n" % pages)
         ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGING, (self,) + pages)
 
-    def ChangePage(self, pgid):
+    def ChangePage(self, pg_num):
         """Change the page and focus to the the given page id
-        @param pgid: Page number to change to
+        @param pg_num: Page number to change 
 
         """
-        window = self.GetPage(pgid) # returns current stc
+        if self.GetSelection() != pg_num:
+            self.SetSelection(pg_num)
+
+        # Get the window that is the current page
+        window = self.GetPage(pg_num)
         window.SetFocus()
         self.control = window
         fname = self.control.GetFileName()
 
+        # Update Frame Title
         if fname == "":
-            fname = self.GetPageText(pgid)
+            fname = self.GetPageText(pg_num)
         self.frame.SetTitle("%s - file://%s" % (util.GetFileName(fname), fname))
+        if not self.frame.IsExiting():
+            ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGED, (self, pg_num))
 
     def OnPageChanged(self, evt):
         """Actions to do after a page change
@@ -518,7 +545,7 @@ class EdPages(FNB.FlatNotebook):
         self.LOG("[ed_pages][info] It has file named: %s" % \
                  self.control.GetFileName())
         self.control.PostPositionEvent()
-        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGED, (self, cpage))
+        self.EnsureVisible(cpage)
 
     def OnPageClosing(self, evt):
         """Checks page status to flag warnings before closing
@@ -593,6 +620,9 @@ class EdPages(FNB.FlatNotebook):
         @param lang_id: language id of file type to get mime image for
 
         """
+        if not Profile_Get('TABICONS'):
+            return
+
         imglst = self.GetImageList()
         if not self._index.has_key(lang_id):
             bmp = wx.ArtProvider.GetBitmap(lang_id, wx.ART_MENU)
@@ -604,29 +634,27 @@ class EdPages(FNB.FlatNotebook):
                                               GetBitmap(lang_id, wx.ART_MENU))
         FNB.FlatNotebook.SetPageImage(self, pg_num, self._index[lang_id])
 
-    def SetSelection(self, pgnum):
-        """Set the currently selected page
-        @param pgnum: page number index
-
-        """
-        csel = self.GetSelection()
-        FNB.FlatNotebook.SetSelection(self, pgnum)
-        self.ChangePage(pgnum)
-
     def UpdateAllImages(self):
         """Reload and Reset all images in the notebook pages and
         the corresponding imagelist to match those of the current theme
         @postcondition: all images in control are updated
 
         """
-        imglst = self.GetImageList()
-        for lang, index in self._index.iteritems():
-            bmp = wx.ArtProvider.GetBitmap(str(lang), wx.ART_MENU)
-            if bmp.IsNull():
-                self._index.setdefault(lang, \
-                                       self._index[synglob.ID_LANG_TXT])
-            else:
-                imglst.Replace(index, bmp)
+        if not Profile_Get('TABICONS'):
+            for page in xrange(self.GetPageCount()):
+                FNB.FlatNotebook.SetPageImage(self, page, -1)
+        else:
+            imglst = self.GetImageList()
+            for lang, index in self._index.iteritems():
+                bmp = wx.ArtProvider.GetBitmap(str(lang), wx.ART_MENU)
+                if bmp.IsNull():
+                    self._index.setdefault(lang, \
+                                           self._index[synglob.ID_LANG_TXT])
+                else:
+                    imglst.Replace(index, bmp)
+
+            for page in xrange(self.GetPageCount()):
+                self.SetPageImage(page, str(self.GetPage(page).GetLangId()))
 
         self.Refresh()
 
@@ -718,7 +746,7 @@ def AskToReload(win, cfile):
             errmap = dict(filename=cfile, errmsg=rmsg)
             mdlg = wx.MessageDialog(win.frame,
                                     _("Failed to reload %(filename)s:\n"
-                                      "Error: %(errmsg)s") % errmsg,
+                                      "Error: %(errmsg)s") % errmap,
                                     _("Error"),
                                     wx.OK | wx.ICON_ERROR)
             mdlg.ShowModal()
