@@ -7,19 +7,16 @@
 ###############################################################################
 
 """
-FILE: ed_statbar.py
-AUTHOR: Cody Precord
-LANGUAGE: Python
-SUMMARY:
-  Custom StatusBar for Editra that contains a progress bar that responds to
+Custom StatusBar for Editra that contains a progress bar that responds to
 messages from ed_msg to display progress of different actions.
 
+@summary: Editra's StatusBar class
 
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_statbar.py 52421 2008-03-10 11:51:19Z CJP $"
-__revision__ = "$Revision: 52421 $"
+__svnid__ = "$Id: ed_statbar.py 54457 2008-07-01 18:24:08Z CJP $"
+__revision__ = "$Revision: 54457 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -48,12 +45,14 @@ class EdStatBar(pstatbar.ProgressStatusBar):
         # Messages
         ed_msg.Subscribe(self.OnProgress, ed_msg.EDMSG_PROGRESS_SHOW)
         ed_msg.Subscribe(self.OnProgress, ed_msg.EDMSG_PROGRESS_STATE)
+        ed_msg.Subscribe(self.OnUpdateText, ed_msg.EDMSG_UI_SB_TXT)
 #        ed_msg.Subscribe(self.OnProgress, ed_msg.EDMSG_FILE_OPENING)
 #        ed_msg.Subscribe(self.OnProgress, ed_msg.EDMSG_FILE_OPENED)
 
     def __del__(self):
         """Unsubscribe from messages"""
         ed_msg.Unsubscribe(self.OnProgress)
+        ed_msg.Unsubscribe(self.OnUpdateText)
         pstatbar.ProgressStatusBar.__del__(self)
 
     def OnProgress(self, msg):
@@ -87,3 +86,13 @@ class EdStatBar(pstatbar.ProgressStatusBar):
             # Data is the file path
             self.SetRange(util.GetFileSize(mdata))
             self.Start(75)
+
+    def OnUpdateText(self, msg):
+        """Update the status bar text based on the recieved message
+        @param msg: Message Object
+
+        """
+        # Only process if this status bar is in the active window and shown
+        if self.GetTopLevelParent().IsActive() and self.IsShown():
+            field, txt = msg.GetData()
+            self.SetStatusText(txt, field)
