@@ -18,8 +18,8 @@ specific options such as commenting code.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 53882 2008-05-31 20:03:29Z CJP $"
-__revision__ = "$Revision: 53882 $"
+__svnid__ = "$Id: ed_stc.py 54784 2008-07-25 13:16:53Z CJP $"
+__revision__ = "$Revision: 54784 $"
 
 #-------------------------------------------------------------------------#
 # Imports
@@ -111,6 +111,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                           keywords=[ ' ' ],
                           syntax_set=list(),
                           comment=list(),
+                          clexer=None,      # Container lexer method
+                          indenter=None,    # Auto indenter
                           lang_id=0)        # Language ID from syntax module
 
         # Set Up Margins
@@ -134,6 +136,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         self.UpdateBaseStyles()
 
         # Other Settings
+        self._menu = MakeMenu()
         self.UsePopUp(False)
 
         #self.Bind(wx.stc.EVT_STC_MACRORECORD, self.OnRecordMacro)
@@ -145,7 +148,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
 
         # Context Menu Events
-        self.Bind(wx.EVT_CONTEXT_MENU, self.OnContextMenu)
+        self.Bind(wx.EVT_CONTEXT_MENU, lambda evt: self.PopupMenu(self._menu))
         self.Bind(wx.EVT_MENU, lambda evt: self.Undo(), id=wx.ID_UNDO)
         self.Bind(wx.EVT_MENU, lambda evt: self.Redo(), id=wx.ID_REDO)
         self.Bind(wx.EVT_MENU, lambda evt: self.Cut(), id=wx.ID_CUT)
@@ -155,7 +158,9 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
        #---- End Init ----#
 
-    __name__ = u"EditraTextCtrl"
+    @property
+    def __name__(self):
+        return u"EditraTextCtrl"
 
     #---- Protected Member Functions ----#
     def _BuildMacro(self):
@@ -243,18 +248,26 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         @postcondition: proper type of white space is added from current pos
                         to match that of indentation in above line
         """
-        line = self.GetCurrentLine()
-        text = self.GetTextRange(self.PositionFromLine(line), \
-                                 self.GetCurrentPos())
-        if text.strip() == u'':
-            self.AddText(self.GetEOLChar() + text)
-            self.EnsureCaretVisible()
-            return
-        indent = self.GetLineIndentation(line)
-        i_space = indent / self.GetTabWidth()
-        ndent = self.GetEOLChar() + self.GetIndentChar() * i_space
-        self.AddText(ndent + \
-                     ((indent - (self.GetTabWidth() * i_space)) * u' '))
+        cpos = self.GetCurrentPos()
+
+        # Check if a special purpose indenter has been registered
+        if self._code['indenter'] is not None:
+            txt = self._code['indenter'](self, cpos, self.GetIndentChar())
+            txt = txt.replace('\n', self.GetEOLChar())
+        else:
+            # Default Indenter
+            line = self.GetCurrentLine()
+            text = self.GetTextRange(self.PositionFromLine(line), cpos)
+            if text.strip() == u'':
+                self.AddText(self.GetEOLChar() + text)
+                self.EnsureCaretVisible()
+                return
+            indent = self.GetLineIndentation(line)
+            i_space = indent / self.GetTabWidth()
+            ndent = self.GetEOLChar() + self.GetIndentChar() * i_space
+            txt = ndent + ((indent - (self.GetTabWidth() * i_space)) * u' ')
+
+        self.AddText(txt)
         self.EnsureCaretVisible()
 
     def Bookmark(self, action):
@@ -472,6 +485,22 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         return self._code['lang_id']
 
+    def GetLineStartPosition(self, line):
+        """Get the starting position of the given line
+        @param line: int
+        @return: int
+
+        """
+        if line > 0:
+            spos = self.GetLineEndPosition(line-1)
+            if self.GetLine(line).endswith("\r\n"):
+                spos += 2
+            else:
+                spos += 1
+        else:
+            spos = 0
+        return spos
+
     def GetLastVisibleLine(self):
         """Return what the last visible line is
         @return: int
@@ -607,11 +636,20 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         Id. If not found it returns an empty string.
         @param style_id: id of tag to look for
         @return: style tag string
+        @todo: change syntax modules to all use ids
 
         """
         for data in self._code['syntax_set']:
-            if style_id == getattr(wx.stc, data[0]):
+            # If its a standard lexer the style id will be stored as
+            # a string. If its a container lexer it is the id.
+            if isinstance(data[0], basestring):
+                s_id = getattr(wx.stc, data[0])
+            else:
+                s_id = data[0]
+
+            if style_id == s_id:
                 return data[1]
+
         return 'default_style'
 
     def GetAutoComplete(self):
@@ -677,15 +715,12 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
         """
         k_code = evt.GetKeyCode()
-        if not evt.ShiftDown() and \
-           self.key_handler.GetHandlerName() == u'VI' and \
-           k_code == wx.WXK_ESCAPE:
-            # If Vi emulation is active go into Normal mode and
-            # pass the key event to OnChar
-            self.key_handler.SetMode(ViKeyHandler.NORMAL)
-            evt.Skip()
-            return
-        elif k_code == wx.WXK_RETURN:
+        if self.key_handler.PreProcessKey(k_code, evt.ControlDown(),
+                                          evt.CmdDown(), evt.ShiftDown(),
+                                          evt.AltDown()):
+            return                
+
+        if k_code == wx.WXK_RETURN:
 
             if self._config['autoindent'] and not self.AutoCompActive():
                 if self.GetSelectedText():
@@ -713,7 +748,9 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         key_code = evt.GetKeyCode()
         cpos = self.GetCurrentPos()
-        if self.key_handler.ProcessKey(key_code):
+        if self.key_handler.ProcessKey(key_code, evt.ControlDown(),
+                                       evt.CmdDown(), evt.ShiftDown(),
+                                       evt.AltDown()):
             # The key handler handled this keypress, we don't need to insert
             # the character into the buffer.
             pass
@@ -740,22 +777,6 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             evt.Skip()
         return
 
-    def OnContextMenu(self, evt):
-        """Show the right click context menu
-        @param evt: EVT_CONTEXT_MENU
-
-        """
-        menu = ed_menu.EdMenu()
-        menu.Append(wx.ID_UNDO, _("Undo"))
-        menu.Append(wx.ID_REDO, _("Redo"))
-        menu.AppendSeparator()
-        menu.Append(wx.ID_CUT, _("Cut"))
-        menu.Append(wx.ID_COPY, _("Copy"))
-        menu.Append(wx.ID_PASTE, _("Paste"))
-        menu.AppendSeparator()
-        menu.Append(wx.ID_SELECTALL, _("Select All"))
-        self.PopupMenu(menu)
-
     def OnKeyUp(self, evt):
         """Update status bar of window
         @param evt: wxEVT_KEY_UP
@@ -793,6 +814,13 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         else:
             evt.Skip()
 
+    def OnStyleNeeded(self, evt):
+        """Perform custom styling when registered for a container lexer"""
+        if self._code['clexer'] is not None:
+            self._code['clexer'](self, self.GetEndStyled(), evt.GetPosition())
+        else:
+            evt.Skip()
+
     def ParaDown(self):
         """Move the caret one paragraph down
         @note: overrides the default function to set caret at end
@@ -1046,7 +1074,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
         # If syntax auto detection fails from file extension try to
         # see if there is an interpreter line that can be parsed.
-        if self.GetLexer() in [0, wx.stc.STC_LEX_NULL]:
+        if self.GetLexer() == wx.stc.STC_LEX_NULL:
             interp = self.GetLine(0)
             if interp != wx.EmptyString:
                 interp = interp.split(u"/")[-1]
@@ -1081,6 +1109,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         e_map = { ed_glob.ID_COPY  : self.Copy, ed_glob.ID_CUT  : self.Cut,
                   ed_glob.ID_PASTE : self.Paste, ed_glob.ID_UNDO : self.Undo,
                   ed_glob.ID_REDO  : self.Redo, ed_glob.ID_INDENT : self.Tab,
+                  ed_glob.ID_REVERT_FILE : self.RevertToSaved,
                   ed_glob.ID_KWHELPER: self.ShowKeywordHelp,
                   ed_glob.ID_CUT_LINE : self.LineCut,
                   ed_glob.ID_COPY_LINE : self.LineCopy,
@@ -1271,7 +1300,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         if self.GetUseTabs():
             return u'\t'
         else:
-            return u' ' * self.GetTabWidth()
+            return u' ' * self.GetIndent()
 
     def GetEOLModeId(self):
         """Gets the id of the eol format
@@ -1411,6 +1440,35 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """Set the buffers filename attributes from the given path"""
         self.file.SetPath(path)
 
+    def SetFocus(self):
+        """Set the focus to the window"""
+        wx.stc.StyledTextCtrl.SetFocus(self)
+        # HACK fix issue where setting the focus requires a
+        # click from the mouse under msw. ?wxBug?
+        if wx.Platform == '__WXMSW__':
+            evt = wx.FocusEvent(wx.wxEVT_SET_FOCUS, self.GetId())
+            wx.PostEvent(self, evt)
+
+    def SetLexer(self, lexer):
+        """Set the buffers lexer
+        @param lexer: lexer to use
+        @note: Overrides StyledTextCtrl.SetLexer
+
+        """
+        if lexer == wx.stc.STC_LEX_CONTAINER:
+            # If setting a container lexer only bind the event if it hasn't
+            # been done yet.
+            if self._code['clexer'] is None:
+                self.Bind(wx.stc.EVT_STC_STYLENEEDED, self.OnStyleNeeded)
+        else:
+            # If changing from a container lexer to a non container
+            # lexer we need to unbind the event.
+            if self._code['clexer'] is not None:
+                self.Unbind(wx.stc.EVT_STC_STYLENEEDED)
+                self._code['clexer'] = None
+
+        wx.stc.StyledTextCtrl.SetLexer(self, lexer)
+
     def SetModTime(self, modtime):
         """Set the value of the files last modtime"""
         self.file.SetModtime(modtime)
@@ -1732,7 +1790,32 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                 return True, ''
         else:
             self.LOG("[ed_stc][err] %s does not exists, cant reload." % cfile)
-            return False, "%s does not exist" % cfile
+            return False, _("%s does not exist") % cfile
+
+    def RevertFile(self):
+        """Revert all the changes made to the file since it was opened
+        @postcondition: undo history is re-wound to initial state and file
+                        is re-saved if it has an on disk file.
+
+        """
+        self.Freeze()
+        while self.CanUndo():
+            self.Undo()
+        self.Thaw()
+
+        fname = self.GetFileName()
+        if len(fname):
+            self.SaveFile(fname)
+
+    def RevertToSaved(self):
+        """Revert the current buffer back to the last save point"""
+        self.Freeze()
+        while self.CanUndo():
+            if self.GetModify():
+                self.Undo()
+            else:
+                break
+        self.Thaw()
 
     def SaveFile(self, path):
         """Save buffers contents to disk
@@ -1811,6 +1894,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
         lexer = syn_data[syntax.LEXER]
         # Check for special cases
+        # TODO: add fetch method to check if container lexer requires extra
+        #       style bytes beyond the default 5.
         if lexer in [ wx.stc.STC_LEX_HTML, wx.stc.STC_LEX_XML]:
             self.SetStyleBits(7)
         elif lexer == wx.stc.STC_LEX_NULL:
@@ -1847,6 +1932,18 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.LOG("[ed_stc][err] No Comment Pattern to set")
             comment = []
 
+        try:
+            clexer = syn_data[syntax.CLEXER]
+        except KeyError:
+            self.LOG("[ed_stc][err] No Container Lexer to set")
+            clexer = None
+
+        try:
+            indenter = syn_data[syntax.INDENTER]
+        except KeyError:
+            self.LOG("[ed_stc][err] No Auto-Indenter")
+            indenter = None
+
         # Set Lexer
         self.SetLexer(lexer)
         # Set Keywords
@@ -1857,6 +1954,10 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         self.SetProperties(props)
         # Set Comment Pattern
         self._code['comment'] = comment
+        # Set the Container Lexer Method
+        self._code['clexer'] = clexer
+        # Auto-indenter function
+        self._code['indenter'] = indenter
 
         # Notify that lexer has changed
         ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_LEXER)
@@ -1902,19 +2003,22 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
                 self.LOG("[ed_stc][warn] Error setting syntax spec")
                 continue
             else:
-                if not isinstance(syn[0], basestring) or \
-                   not hasattr(wx.stc, syn[0]):
-                    self.LOG("[ed_stc][warn] Unknown syntax region: %s" % \
-                             str(syn[0]))
-                    continue
-                elif not isinstance(syn[1], basestring):
-                    self.LOG("[ed_stc][warn] Poorly formated styletag: %s" % \
-                             str(syn[1]))
-                    continue
+                if self.GetLexer() == wx.stc.STC_LEX_CONTAINER:
+                    self.StyleSetSpec(syn[0], self.GetStyleByName(syn[1]))
                 else:
-                    self.StyleSetSpec(getattr(wx.stc, syn[0]), \
-                                      self.GetStyleByName(syn[1]))
-                    valid_settings.append(syn)
+                    if not isinstance(syn[0], basestring) or \
+                       not hasattr(wx.stc, syn[0]):
+                        self.LOG("[ed_stc][warn] Unknown syntax region: %s" % \
+                                 str(syn[0]))
+                        continue
+                    elif not isinstance(syn[1], basestring):
+                        self.LOG("[ed_stc][warn] Poorly formated styletag: %s" % \
+                                 str(syn[1]))
+                        continue
+                    else:
+                        self.StyleSetSpec(getattr(wx.stc, syn[0]), \
+                                          self.GetStyleByName(syn[1]))
+                valid_settings.append(syn)
         self._code['syntax_set'] = valid_settings
         return True
 
@@ -2019,3 +2123,16 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
     #---- End Style Definitions ----#
 
 #-----------------------------------------------------------------------------#
+
+def MakeMenu():
+    """Make the buffers context menu"""
+    menu = ed_menu.EdMenu()
+    menu.Append(wx.ID_UNDO, _("Undo"))
+    menu.Append(wx.ID_REDO, _("Redo"))
+    menu.AppendSeparator()
+    menu.Append(wx.ID_CUT, _("Cut"))
+    menu.Append(wx.ID_COPY, _("Copy"))
+    menu.Append(wx.ID_PASTE, _("Paste"))
+    menu.AppendSeparator()
+    menu.Append(wx.ID_SELECTALL, _("Select All"))
+    return menu
