@@ -8,21 +8,20 @@
 ###############################################################################
 
 """
-FILE: ed_style.py                                                        
-AUTHOR: Cody Precord                                                     
-LANGUAGE: Python                                                         
-SUMMARY:                                                                 
-  Provides a system for managing styles in the text control. Compiles the data
+Provides a system for managing styles in the text control. Compiles the data
 in an Editra Style Sheet to a format that Scintilla can understand. The
 specification of Editra Style Sheets that this module implements can be found
 either in the _docs_ folder of the source distribution or on Editra's home page
-http://editra.org/?page=docs&doc=ess_spec.
-                                                                      
+U{http://editra.org/?page=docs&doc=ess_spec}.
+
+@summary: Style management system for managing the syntax highlighting of all
+          buffers
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_style.py 51983 2008-02-22 07:59:28Z CJP $"
-__revision__ = "$Revision: 51983 $"
+__svnid__ = "$Id: ed_style.py 54987 2008-08-05 23:18:42Z CJP $"
+__revision__ = "$Revision: 54987 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -33,7 +32,7 @@ import util
 from profiler import Profile_Get, Profile_Set
 
 # Globals
-STY_ATTRIBUTES     = u"face fore back size"
+STY_ATTRIBUTES     = u"face fore back size modifiers"
 STY_EX_ATTRIBUTES  = u"eol bold italic underline"
 
 # Parser Values
@@ -47,34 +46,44 @@ class StyleItem(object):
     """A storage class for holding styling information
     @todo: The extra Attributes should be saved as a separate attribute in the
            StyleItem. This currenlty causes problems when customizing values in
-           the StyleEditor. Changing this is fairly easy in this class but it 
+           the StyleEditor. Changing this is fairly easy in this class but it
            will require changes to the StyleMgr and Editor as well.
 
     """
-    def __init__(self, fore=wx.EmptyString, back=wx.EmptyString, 
-                       face=wx.EmptyString, size=wx.EmptyString):
+    def __init__(self, fore=wx.EmptyString, back=wx.EmptyString,
+                       face=wx.EmptyString, size=wx.EmptyString,
+                       ex=list()):
         """Initiliazes the Style Object.
 
         @keyword fore: Specifies the forground color (hex string)
         @keyword face: Specifies the font face (string face name)
         @keyword back: Specifies the background color (hex string)
         @keyword size: Specifies font point size (int/formatted string)
+        @keyword ex: Specify modifiers
 
         SPECIFICATION:
-          DATA FORMATS:
-            #123456       = hex color code
-            #123456,bold  = hex color code + extra style
-            Monaco        = Font Face Name
-            %(primary)s   = Format string to be swapped at runtime
-            10            = A font point size
-            %(size)s      = Format string to be swapped at runtime
+          - DATA FORMATS:
+            - #123456       = hex color code
+            - Monaco        = Font Face Name
+            - %(primary)s   = Format string to be swapped at runtime
+            - 10            = A font point size
+            - %(size)s      = Format string to be swapped at runtime
+            - ex            = bold underline italic eol
+
         """
         object.__init__(self)
+
+        # Attributes
         self.null = False
-        self.fore = fore        # Foreground color hex code
-        self.face = face        # Font face name
-        self.back = back        # Background color hex code
-        self.size = size        # Font point size
+        self.fore = u""        # Foreground color hex code
+        self.face = u""        # Font face name
+        self.back = u""        # Background color hex code
+        self.size = u""        # Font point size
+        self._exattr = list()   # Extra attributes
+
+        for attr, value in (("fore", fore), ("face", face),
+                            ("back", back), ("size", size)):
+            self.SetNamedAttr(attr, value)
 
     def __eq__(self, si2):
         """Defines the == operator for the StyleItem Class
@@ -87,24 +96,25 @@ class StyleItem(object):
 
     def __str__(self):
         """Converts StyleItem to a string
-        @note: this return string is in a format that can be accepted by
+        @note: This return string is in a format that can be accepted by
                Scintilla. No spaces may be in the string after the ':'.
         @return: string representation of the StyleItem
 
         """
-        style_str = wx.EmptyString
+        style_str = list()
         if self.fore:
-            style_str = u"fore:%s," % self.fore
+            style_str.append(u"fore:%s" % self.fore)
         if self.back:
-            style_str += u"back:%s," % self.back
+            style_str.append(u"back:%s" % self.back)
         if self.face:
-            style_str += u"face:%s," % self.face
+            style_str.append(u"face:%s" % self.face)
         if self.size:
-            style_str += u"size:%s," % str(self.size)
+            style_str.append(u"size:%s" % str(self.size))
+        if len(self._exattr): 
+            style_str.append(u"modifiers:" +  u','.join(self._exattr))
 
-        if len(style_str) and style_str[-1] == u',':
-            style_str = style_str[0:-1]
-        return style_str
+        style_str = u",".join(style_str)
+        return style_str.rstrip(u",")
 
     #---- Get Functions ----#
     def GetAsList(self):
@@ -118,6 +128,9 @@ class StyleItem(object):
             val = getattr(self, attr, None)
             if val not in ( None, wx.EmptyString ):
                 retval.append(attr + ':' + val)
+
+        if len(self._exattr):
+            retval.append("modifiers:" + u",".join(self._exattr))
         return retval
 
     def GetBack(self):
@@ -125,28 +138,42 @@ class StyleItem(object):
         @return: style items background attribute
 
         """
-        return self.back.split(',')[0]
+        return self.back
 
     def GetFace(self):
         """Returns the value of the face attribute
         @return: style items font face attribute
 
         """
-        return self.face.split(',')[0]
+        return self.face
 
     def GetFore(self):
         """Returns the value of the fore attribute
         @return: style items foreground attribute
 
         """
-        return self.fore.split(',')[0]
+        return self.fore
 
     def GetSize(self):
         """Returns the value of the size attribute as a string
         @return: style items font size attribute
 
         """
-        return self.size.split(',')[0]
+        return self.size
+
+    def GetModifiers(self):
+        """Get the modifiers string
+        @return: string
+
+        """
+        return ",".join(self.GetModifierList())
+
+    def GetModifierList(self):
+        """Get the list of modifiers
+        @return: list
+
+        """
+        return self._exattr
 
     def GetNamedAttr(self, attr):
         """Get the value of the named attribute
@@ -179,11 +206,12 @@ class StyleItem(object):
         self.null = True
         for attr in ('fore', 'face', 'back', 'size'):
             setattr(self, attr, '')
+        self._exattr = list()
 
     #---- Set Functions ----#
     def SetAttrFromStr(self, style_str):
         """Takes style string and sets the objects attributes
-        by parsing the string for the values. Only sets or 
+        by parsing the string for the values. Only sets or
         overwrites values does not zero out previously set values.
         Returning True if value(s) are set or false otherwise.
         @param style_str: style information string (i.e fore:#888444)
@@ -196,12 +224,14 @@ class StyleItem(object):
             attrib = atom.split(u':')
             if len(attrib) == 2 and attrib[0] in STY_ATTRIBUTES:
                 last_set = attrib[0]
-                setattr(self, attrib[0], attrib[1])
-            elif attrib[0] in STY_EX_ATTRIBUTES and last_set != wx.EmptyString:
-                l_val = getattr(self, last_set)
-                setattr(self, last_set, u",".join([l_val, attrib[0]]))
+                if last_set == "modifiers":
+                    self.SetExAttr(attrib[1])
+                else:
+                    setattr(self, attrib[0], attrib[1])
             else:
-                pass
+                for attr in attrib:
+                    if attr in STY_EX_ATTRIBUTES:
+                        self.SetExAttr(attr)
 
         return last_set != wx.EmptyString
 
@@ -212,10 +242,9 @@ class StyleItem(object):
 
         """
         self.null = False
-        if back is None or ex == wx.EmptyString:
-            self.back = back
-        else:
-            self.back = u"%s,%s" % (back, ex)
+        self.back = back
+        if ex and ex not in self._exattr:
+            self._exattr.append(ex)
 
     def SetFace(self, face, ex=wx.EmptyString):
         """Sets the Face Value
@@ -223,10 +252,9 @@ class StyleItem(object):
         @keyword ex: extra attribute (i.e bold, italic, underline)
 
         """
-        if face is None or ex == wx.EmptyString:
-            self.face = face
-        else:
-            self.face = u"%s,%s" % (face, ex)
+        self.face = face
+        if ex and ex not in self._exattr:
+            self._exattr.append(ex)
 
     def SetFore(self, fore, ex=wx.EmptyString):
         """Sets the Foreground Value
@@ -235,10 +263,9 @@ class StyleItem(object):
 
         """
         self.null = False
-        if fore is None or ex == wx.EmptyString:
-            self.fore = fore
-        else:
-            self.fore = u"%s,%s" % (fore, ex)
+        self.fore = fore
+        if ex and ex not in self._exattr:
+            self._exattr.append(ex)
 
     def SetSize(self, size, ex=wx.EmptyString):
         """Sets the Font Size Value
@@ -248,10 +275,9 @@ class StyleItem(object):
 
         """
         self.null = False
-        if size is None or ex == wx.EmptyString:
-            self.size = size
-        else:
-            self.size = u"%s,%s" % (str(size), ex)
+        self.size = str(size)
+        if ex and ex not in self._exattr:
+            self._exattr.append(ex)
 
     def SetExAttr(self, ex_attr, add=True):
         """Adds an extra text attribute to a StyleItem. Currently
@@ -265,27 +291,19 @@ class StyleItem(object):
         """
         # Get currently set attributes
         self.null = False
-        cur_str = self.__str__()
-        if not add:
-            cur_str = cur_str.replace(u',' + ex_attr, wx.EmptyString)
-            self.SetAttrFromStr(cur_str)
+        if ex_attr not in STY_EX_ATTRIBUTES:
+            return
+
+        if add and ex_attr not in self._exattr:
+            self._exattr.append(ex_attr)
+        elif not add and ex_attr in self._exattr:
+            self._exattr.remove(ex_attr)
         else:
-            if u',' + ex_attr not in cur_str:
-                attr_map = { u"fore" : self.GetFore(),
-                             u"back" : self.GetBack(),
-                             u"face" : self.GetFace(),
-                             u"size" : self.GetSize()
-                           }
-                for key in attr_map:
-                    if len(attr_map[key]) and u"," not in attr_map[key]:
-                        setattr(self, key, u",".join([attr_map[key], ex_attr]))
-                        break
-            else:
-                pass
+            pass
 
     def SetNamedAttr(self, attr, value):
         """Sets a StyleItem attribute by named string.
-        @note: This is not intended to be used for setting extra 
+        @note: This is not intended to be used for setting extra
                attributes such as bold, eol, ect..
         @param attr: a particular attribute to set (i.e fore, face, back, size)
         @param value: value to set the attribute to contain
@@ -294,17 +312,18 @@ class StyleItem(object):
         self.null = False
         cur_val = getattr(self, attr, None)
         if cur_val is not None:
-            if u"," in cur_val:
-                tmp = cur_val.split(u",")
-                tmp[0] = value
-                value = u",".join(tmp)
-        setattr(self, attr, value)
+            if u"," in value:
+                modifiers = value.split(u",")
+                value = modifiers.pop(0)
+                for ex in modifiers:
+                    self.SetExAttr(ex)
+            setattr(self, attr, value)
 
 #-----------------------------------------------------------------------------#
 
 class StyleMgr(object):
     """Manages style definitions and provides them on request.
-    Also provides functionality for loading custom style sheets and 
+    Also provides functionality for loading custom style sheets and
     modifying styles during run time.
 
     """
@@ -330,6 +349,8 @@ class StyleMgr(object):
         # Get the Style Set
         if custom != wx.EmptyString and self.LoadStyleSheet(custom):
             self.LOG("[ed_style][info] Loaded custom style sheet %s" % custom)
+        elif custom == wx.EmptyString:
+            self.SetStyles('default', DefaultStyleDictionary())
         else:
             self.LOG("[ed_style][err] Failed to import styles from %s" % custom)
 
@@ -341,11 +362,11 @@ class StyleMgr(object):
 
         """
         sty_dict = dict()
-        for key in self.GetStyleSet().keys():
+        for key in DefaultStyleDictionary().keys():
             if key in ('select_style', 'whitespace_style'):
                 sty_dict[key] = NullStyleItem()
             else:
-                sty_dict[key] = StyleItem("#000000", "#FFFFFF", 
+                sty_dict[key] = StyleItem("#000000", "#FFFFFF",
                                           "%(primary)s", "%(size)d")
         return sty_dict
 
@@ -365,18 +386,18 @@ class StyleMgr(object):
         if font is not None:
             mfont = font
         else:
-            mfont = wx.Font(10, wx.FONTFAMILY_MODERN, wx.FONTSTYLE_NORMAL, 
+            mfont = wx.Font(10, wx.FONTFAMILY_MODERN, wx.FONTSTYLE_NORMAL,
                             wx.FONTWEIGHT_NORMAL)
             Profile_Set('FONT1', mfont, 'font')
         primary = mfont.GetFaceName()
 
         font = Profile_Get('FONT2', 'font', None)
         if font is None:
-            font = wx.Font(10, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, 
+            font = wx.Font(10, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL,
                             wx.FONTWEIGHT_NORMAL)
             Profile_Set('FONT2', font, 'font')
         secondary = font.GetFaceName()
-        faces = { 
+        faces = {
                   self.FONT_PRIMARY   : primary,
                   self.FONT_SECONDARY : secondary,
                   self.FONT_SIZE  : mfont.GetPointSize(),
@@ -424,6 +445,13 @@ class StyleMgr(object):
             fore = wx.Colour(red=rgb[0], green=rgb[1], blue=rgb[2])
         return fore
 
+    def GetCurrentStyleSetName(self):
+        """Get the name of the currently set style
+        @return: string
+
+        """
+        return self.style_set
+
     def GetDefaultBackColour(self, as_hex=False):
         """Gets the background color of the default style and returns
         a Colour object. Otherwise returns white if the default
@@ -431,7 +459,7 @@ class StyleMgr(object):
         @keyword hex: return a hex string or colour object
         @type hex: bool
         @return: wx.Colour of default style background or hex value
-        @rtype: wx.Colour or string 
+        @rtype: wx.Colour or string
 
         """
         back = self.GetItemByName('default_style').GetBack()
@@ -455,9 +483,9 @@ class StyleMgr(object):
                 val = unicode(StyleMgr.STYLES[self.style_set][name]) % self.fonts
                 item = StyleItem()
                 item.SetAttrFromStr(val)
-                return item
             else:
-                return StyleMgr.STYLES[self.style_set][name]
+                item = StyleMgr.STYLES[self.style_set][name]
+            return item
         else:
             return StyleItem()
 
@@ -468,10 +496,10 @@ class StyleMgr(object):
 
         """
         if primary:
-            font = wx.FFont(self.fonts[self.FONT_SIZE], wx.DEFAULT, 
+            font = wx.FFont(self.fonts[self.FONT_SIZE], wx.DEFAULT,
                             face=self.fonts[self.FONT_PRIMARY])
         else:
-            font = wx.FFont(self.fonts[self.FONT_SIZE2], wx.DEFAULT, 
+            font = wx.FFont(self.fonts[self.FONT_SIZE2], wx.DEFAULT,
                             face=self.fonts[self.FONT_SECONDARY])
         return font
 
@@ -484,12 +512,13 @@ class StyleMgr(object):
 
         """
         if self.HasNamedStyle(name):
-            return unicode(self.GetItemByName(name))
+            stystr = unicode(self.GetItemByName(name))
+            return stystr.replace("modifiers:", "")
         else:
             return wx.EmptyString
 
     def GetStyleSet(self):
-        """Returns the current set of styles or the default set if 
+        """Returns the current set of styles or the default set if
         there is no current set.
         @return: current style set dictionary
         @rtype: dict
@@ -527,6 +556,8 @@ class StyleMgr(object):
             return ret_val
         elif not StyleMgr.STYLES.has_key(style_sheet):
             self.LOG("[ed_style][warn] Style sheet %s does not exists" % style_sheet)
+            # Reset to default style
+            Profile_Set('SYNTHEME', 'default')
             self.SetStyles('default', DefaultStyleDictionary())
             return False
         else:
@@ -600,7 +631,7 @@ class StyleMgr(object):
         # Tree Level 2 Build small trees of tag and style attributes
         # Tree Level 3 Branch tree into TAG => Attr => Value String
         for branch in style_tree:
-            tmp2 = [leaf.strip().split(u":") 
+            tmp2 = [leaf.strip().split(u":")
                     for leaf in branch[1].strip().split(u";")]
             if len(tmp2) and tmp2[-1][0] == wx.EmptyString:
                 tmp2.pop()
@@ -634,8 +665,9 @@ class StyleMgr(object):
         for style_def in tmp:
             if len(tmp[style_def]) == 0:
                 style_dict.pop(style_def)
+        del tmp
 
-        # Validate leaf values and format into stylestring
+        # Validate leaf values and format into style string
         for style_def in style_dict:
             if not style_def[0][0].isalpha():
                 self.LOG("[ed_style][err] The style def %s is not a "
@@ -644,18 +676,13 @@ class StyleMgr(object):
                     raise SyntaxError, "%s is an invalid name" % style_def[0]
             else:
                 style_str = wx.EmptyString
+                # Check each definition and validate its items
                 for attrib in style_dict[style_def]:
-                    values = [ val for val in attrib[1].split(u" ") 
+                    values = [ val for val in attrib[1].split()
                                if val != wx.EmptyString ]
-                    if len(values) > 2:
-                        self.LOG("[ed_style][warn] Only one extra " +
-                                 "attribute can be set per style. See " +
-                                 style_def + " => " + attrib[0])
-                        if strict:
-                            raise SyntaxWarning
 
-                    # Validate values
                     v1ok = v2ok = False
+                    # Check that colors are a hex string
                     if attrib[0] in "fore back" and RE_HEX_STR.match(values[0]):
                         v1ok = True
                     elif len(values) and attrib[0] == "size":
@@ -666,17 +693,31 @@ class StyleMgr(object):
                                      " the value %s is invalid." % \
                                      (attrib[0], values[0]))
                     elif len(values) and attrib[0] == "face":
-                        if len(values) == 2 and \
-                           values[1] not in STY_EX_ATTRIBUTES:
-                            values = [u' '.join(values)]
+                        # Font names may have spaces in them so join the
+                        # name of the font into one item.
+                        if len(values) > 1 and values[1] not in STY_EX_ATTRIBUTES:
+                            tmp = list()
+                            for val in list(values):
+                                if val not in STY_EX_ATTRIBUTES:
+                                    tmp.append(val)
+                                    values.remove(val)
+                                else:
+                                    break
+                            values = [u' '.join(tmp),] + values
+                        v1ok = True
+                    elif len(values) and attrib[0] == "modifiers":
                         v1ok = True
 
-                    if len(values) == 2 and values[1] in STY_EX_ATTRIBUTES:
-                        v2ok = True
-                    elif len(values) == 2:
-                        self.LOG("[ed_style][warn] Unknown extra " + \
-                                 "attribute '" + values[1] + \
-                                 "' in attribute: " + attrib[0])
+                    # Check extra attributes
+                    if len(values) > 1:
+                        for value in values[1:]:
+                            if value not in STY_EX_ATTRIBUTES:
+                                self.LOG("[ed_style][warn] Unknown extra " + \
+                                         "attribute '" + values[1] + \
+                                         "' in attribute: " + attrib[0])
+                                break
+                            else:
+                                v2ok = True
 
                     if v1ok and v2ok:
                         value = u",".join(values)
@@ -684,8 +725,8 @@ class StyleMgr(object):
                         value = values[0]
                     else:
                         continue
-                    
-                    style_str = u",".join([style_str, 
+
+                    style_str = u",".join([style_str,
                                            u":".join([attrib[0], value])])
 
                 if style_str != wx.EmptyString:
@@ -733,9 +774,14 @@ class StyleMgr(object):
         """Sets the value of style tag by name
         @param style_tag: desired tag name of style definition
         @param value: style item to set tag to
+        @return: bool
 
         """
+        if not isinstance(value, StyleItem):
+            return False
+
         StyleMgr.STYLES[self.style_set][style_tag] = value
+        return True
 
     def SetStyles(self, name, style_dict, nomerge=False):
         """Sets the managers style data and returns True on success.
@@ -761,8 +807,20 @@ class StyleMgr(object):
                     return False
 
             self.style_set = name
-            tmp = MergeStyles(DefaultStyleDictionary(), style_dict)
-            StyleMgr.STYLES[name] = self.PackStyleSet(tmp)
+            defaultd = DefaultStyleDictionary()
+            dstyle = style_dict.get('default_style', None)
+            if dstyle is None:
+                style_dict['default_style'] = defaultd['default_style']
+
+            # Set any undefined styles to match the default_style
+            for tag, item in defaultd.iteritems():
+                if tag not in style_dict:
+                    if tag in ['select_style', 'whitespace_style']:
+                        style_dict[tag] = NullStyleItem()
+                    else:
+                        style_dict[tag] = style_dict['default_style']
+
+            StyleMgr.STYLES[name] = self.PackStyleSet(style_dict)
             return True
         else:
             self.LOG("[ed_style][err] SetStyles expects a " \
@@ -773,55 +831,66 @@ class StyleMgr(object):
 # Utility Functions
 def DefaultStyleDictionary():
     """This is the default style values that are used for styling
-    documents. Its used as a fallback for undefined values in a 
+    documents. Its used as a fallback for undefined values in a
     style sheet.
     @note: incomplete style sheets are merged against this set to ensure
            a full set of definitions is avaiable
 
     """
     def_dict = \
-        {'brace_good' : StyleItem("#FFFFFF", "#0000FF,bold"),
-         'brace_bad'  : StyleItem(back="#FF0000,bold"),
+        {'brace_good' : StyleItem("#FFFFFF", "#0000FF", ex=["bold",]),
+         'brace_bad'  : StyleItem(back="#FF0000", ex=["bold",]),
          'calltip'    : StyleItem("#404040", "#FFFFB8"),
+         'caret_line' : StyleItem(back="#D8F8FF"),
          'ctrl_char'  : StyleItem(),
          'line_num'   : StyleItem(back="#C0C0C0", face="%(secondary)s", \
                                   size="%(size3)d"),
-         'array_style': StyleItem("#EE8B02,bold", face="%(secondary)s"),
-         'btick_style': StyleItem("#8959F6,bold", size="%(size)d"),
+         'array_style': StyleItem("#EE8B02",
+                                  face="%(secondary)s",
+                                  ex=["bold",]),
+         'btick_style': StyleItem("#8959F6", size="%(size)d", ex=["bold",]),
          'default_style': StyleItem("#000000", "#F6F6F6", \
                                     "%(primary)s", "%(size)d"),
          'char_style' : StyleItem("#FF3AFF"),
-         'class_style' : StyleItem("#2E8B57,bold"),
-         'class2_style' : StyleItem("#2E8B57,bold"),
+         'class_style' : StyleItem("#2E8B57", ex=["bold",]),
+         'class2_style' : StyleItem("#2E8B57", ex=["bold",]),
          'comment_style' : StyleItem("#838383"),
-         'decor_style' : StyleItem("#BA0EEA italic", face="%(secondary)s"),
-         'directive_style' : StyleItem("#0000FF,bold", face="%(secondary)s"),
+         'decor_style' : StyleItem("#BA0EEA", face="%(secondary)s",
+                                   ex=["italic",]),
+         'directive_style' : StyleItem("#0000FF", face="%(secondary)s",
+                                       ex=["bold",]),
          'dockey_style' : StyleItem("#0000FF"),
-         'error_style' : StyleItem("#DD0101,bold", face="%(secondary)s"),
-         'funct_style' : StyleItem("#008B8B,italic"),
-         'global_style' : StyleItem("#007F7F,bold", face="%(secondary)s"),
+         'error_style' : StyleItem("#DD0101", face="%(secondary)s",
+                                    ex=["bold",]),
+         'foldmargin_style' : StyleItem(back="#D1D1D1"),
+         'funct_style' : StyleItem("#008B8B", ex=["italic",]),
+         'global_style' : StyleItem("#007F7F", face="%(secondary)s",
+                                    ex=["bold",]),
          'guide_style' : StyleItem("#838383"),
-         'here_style' : StyleItem("#CA61CA,bold", face="%(secondary)s"),
+         'here_style' : StyleItem("#CA61CA", face="%(secondary)s",
+                                  ex=["bold",]),
          'ideol_style' : StyleItem("#E0C0E0", face="%(secondary)s"),
-         'keyword_style' : StyleItem("#A52B2B,bold"),
-         'keyword2_style' : StyleItem("#2E8B57,bold"),
-         'keyword3_style' : StyleItem("#008B8B,bold"),
+         'keyword_style' : StyleItem("#A52B2B", ex=["bold",]),
+         'keyword2_style' : StyleItem("#2E8B57", ex=["bold",]),
+         'keyword3_style' : StyleItem("#008B8B", ex=["bold",]),
          'keyword4_style' : StyleItem("#9D2424"),
          'marker_style' : StyleItem("#FFFFFF", "#000000"),
-         'folder_style' : StyleItem("#FFFFFF", "#000000"),
          'number_style' : StyleItem("#DD0101"),
-         'number2_style' : StyleItem("#DD0101,bold"),
-         'operator_style' : StyleItem("#000000", face="%(primary)s,bold"),
-         'pre_style' : StyleItem("#AB39F2,bold"),               
-         'pre2_style' : StyleItem("#AB39F2,bold", "#FFFFFF"),
+         'number2_style' : StyleItem("#DD0101", ex=["bold",]),
+         'operator_style' : StyleItem("#000000", face="%(primary)s",
+                                      ex=["bold",]),
+         'pre_style' : StyleItem("#AB39F2", ex=["bold",]),
+         'pre2_style' : StyleItem("#AB39F2", "#FFFFFF", ex=["bold",]),
          'regex_style' : StyleItem("#008B8B"),
-         'scalar_style' : StyleItem("#AB37F2,bold", face="%(secondary)s"),
+         'scalar_style' : StyleItem("#AB37F2", face="%(secondary)s",
+                                    ex=["bold",]),
          'scalar2_style' : StyleItem("#AB37F2", face="%(secondary)s"),
          'select_style' : NullStyleItem(), # Use system default colour
-         'string_style' : StyleItem("#FF3AFF,bold"),
-         'stringeol_style' : StyleItem("#000000,bold", "#EEC0EE,eol", \
-                                       "%(secondary)s"),
-         'unknown_style' : StyleItem("#FFFFFF,bold", "#DD0101,eol"),
+         'string_style' : StyleItem("#FF3AFF", ex=["bold",]),
+         'stringeol_style' : StyleItem("#000000", "#EEC0EE", \
+                                       "%(secondary)s", ["bold", "eol"]),
+         'unknown_style' : StyleItem("#FFFFFF", "#DD0101", ex=["bold", "eol"]),
+         'userkw_style' : StyleItem(),
          'whitespace_style' : StyleItem('#838383')
          }
     return def_dict
