@@ -7,20 +7,18 @@
 ###############################################################################
 
 """
-FILE: ed_txt.py
-AUTHOR: Cody Precord
-LANGUAGE: Python
-SUMMARY: Text/Unicode handling functions and File wrapper class
+Text/Unicode handling functions and File wrapper class
 
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_txt.py 52380 2008-03-08 10:30:35Z CJP $"
-__revision__ = "$Revision: 52380 $"
+__svnid__ = "$Id: ed_txt.py 55014 2008-08-07 19:39:33Z CJP $"
+__revision__ = "$Revision: 55014 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import os
+import sys
 import re
 import codecs
 import locale
@@ -36,11 +34,11 @@ from util import Log
 # Default encoding to use when nothing is specified
 # Use utf-8 where available else fallback to the system
 # encoding
-DEFAULT_ENCODING = 'utf-8'
+DEFAULT_ENCODING = locale.getpreferredencoding()
 try:
     codecs.lookup(DEFAULT_ENCODING)
 except LookupError:
-    DEFAULT_ENCODING = locale.getpreferredencoding()
+    DEFAULT_ENCODING = 'utf-8'
 
 # File Helper Functions
 BOM = { 'utf-8' : codecs.BOM_UTF8,
@@ -315,7 +313,7 @@ class EdFile(object):
 # Utility Function
 def CheckBom(line):
     """Try to look for a bom byte at the begining of the given line
-    @param txt: line (first line) of a file
+    @param line: line (first line) of a file
     @return: encoding or None
 
     """
@@ -357,7 +355,7 @@ def DecodeString(string, encoding=None):
     if not encoding:
         encoding = DEFAULT_ENCODING
 
-    if isinstance(string, types.UnicodeType):
+    if not isinstance(string, types.UnicodeType):
         try:
             rtxt = codecs.getdecoder(encoding)(string)[0]
         except Exception, msg:
@@ -366,6 +364,7 @@ def DecodeString(string, encoding=None):
             rtxt = string
         return rtxt
     else:
+        # The string is already unicode so just return it
         return string
 
 def EncodeString(string, encoding=None):
@@ -374,7 +373,7 @@ def EncodeString(string, encoding=None):
     default encoding will be used if None is given for the
     encoding.
     @param string: unicode object to encode into a string
-    @keyword: encoding to use for conversion
+    @keyword encoding: encoding to use for conversion
 
     """
     if not encoding:
@@ -390,13 +389,13 @@ def EncodeString(string, encoding=None):
         return string
 
 def FallbackReader(fname):
-    """Guess then encoding of a file by brute force by trying one
+    """Guess the encoding of a file by brute force by trying one
     encoding after the next until something succeeds.
     @param fname: file path to read from
 
     """
     txt = None
-    for enc in ('utf-8', 'utf-16', 'latin-1', 'ascii'):
+    for enc in GetEncodings():
         try:
             handle = open(fname, 'rb')
             reader = codecs.getreader(enc)(handle)
@@ -409,3 +408,39 @@ def FallbackReader(fname):
             return (enc, txt)
 
     return (None, None)
+
+def GetEncodings():
+    """Get a list of possible encodings to try from the locale information
+    @return: list of strings
+
+    """
+    encodings = list()
+    try:
+        encodings.append(locale.getpreferredencoding())
+    except:
+        pass
+    
+    encodings.append('utf-8')
+
+    try:
+        encodings.append(locale.nl_langinfo(locale.CODESET))
+    except:
+        pass
+    try:
+        encodings.append(locale.getlocale()[1])
+    except:
+        pass
+    try:
+        encodings.append(locale.getdefaultlocale()[1])
+    except:
+        pass
+    encodings.append(sys.getfilesystemencoding())
+    encodings.append('latin-1')
+
+    # Clean the list for duplicates and None values
+    rlist = list()
+    for enc in encodings:
+        if enc is not None and len(enc) and enc not in rlist:
+            rlist.append(enc.lower())
+
+    return rlist
