@@ -4,40 +4,38 @@
 #          from the contents of a EdStc text buffer (i.e HTML, LaTeX, Rtf)    #
 # Author: Cody Precord <cprecord@editra.org>                                  #
 # Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE: generator.py                                                       #
-# AUTHOR: Cody Precord                                                     #
-# LANGUAGE: Python                                                         #
-# SUMMARY:                                                                 #
-#    Provides various methods and classes for generating code and          #
-# transforming code to different formats such as html, latex, rtf with all #
-# the styling and formating intact from how the view is shown in the       #
-# editor.                                                                  #
-#                                                                          #
-#    It also provides a plugin interface that allows for plugins that wish #
-# to provide similar services for manipulating and transforming text.      #
-#                                                                          #
-#--------------------------------------------------------------------------#
+Provides various methods and classes for generating code and transforming code
+to different formats such as html, latex, rtf with all the styling and formating
+intact from how the view is shown in the editor.
+
+It also provides a plugin interface that allows for plugins that wish to provide
+similar services for manipulating and transforming text.
+
+@summary: Editra's Generator interface and implementations
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: generator.py 51110 2008-01-08 15:03:38Z CJP $"
-__revision__ = "$Revision: 51110 $"
+__svnid__ = "$Id: generator.py 54989 2008-08-05 23:36:28Z CJP $"
+__revision__ = "$Revision: 54989 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import wx
 import wx.stc
+import time
+
+# Editra Libraries
 import ed_glob
 import ed_menu
 from ed_style import StyleItem
 import util
 import plugin
-import time
+
 #--------------------------------------------------------------------------#
 # Globals
 _ = wx.GetTranslation
@@ -80,6 +78,8 @@ class GeneratorI(plugin.Interface):
         """
         pass
 
+#-----------------------------------------------------------------------------#
+
 class Generator(plugin.Plugin):
     """Plugin Interface Extension Point for Generator
     type plugin objects. Generator objects are used
@@ -119,7 +119,7 @@ class Generator(plugin.Plugin):
         @type txt_ctrl: EditraStc
         @return: the generated text
         @rtype: string
-        
+
         """
         gentext = None
         start = time.time()
@@ -129,12 +129,12 @@ class Generator(plugin.Plugin):
         util.Log("[generator][info] Generation time %f" % (time.time() - start))
         return gentext
 
-#--------------------------------------------------------------------------#
+#-----------------------------------------------------------------------------#
 
 class Html(plugin.Plugin):
     """Transforms the text from a given Editra stc to a fully
     styled html page. Inline CSS is generated and inserted into
-    the head of the Html to style the text regions by default 
+    the head of the Html to style the text regions by default
     unless requested to generate a separate sheet.
 
     """
@@ -196,7 +196,7 @@ class Html(plugin.Plugin):
                "<meta name=\"Generator\" content=\"Editra/%s\">\n" \
                "<meta http-equiv=\"content-type\" content=\"text/html; " \
                "charset=utf-8\">" \
-               "\n</head>" % (util.GetFileName(self.stc.GetFileName()), 
+               "\n</head>" % (util.GetFileName(self.stc.GetFileName()),
                               ed_glob.VERSION)
 
     def GenerateBody(self):
@@ -220,7 +220,7 @@ class Html(plugin.Plugin):
             s_item = StyleItem()
             s_item.SetAttrFromStr(self.stc.GetStyleByName(tag))
             self.css[tag] = CssItem(tag.split('_')[0], s_item)
- 
+
         # Optimizations
         stc = self.stc
         GetStyleAt = stc.GetStyleAt
@@ -315,6 +315,8 @@ class Html(plugin.Plugin):
         text = text.replace("\"", "&quot;")
         return text
 
+#-----------------------------------------------------------------------------#
+
 class CssItem(object):
     """Converts an Edtira StyleItem to a Css item for use in
     generating html.
@@ -332,6 +334,8 @@ class CssItem(object):
 
         """
         object.__init__(self)
+
+        # Attributes
         self._tag = class_tag
         self._back = style_item.GetBack()
         self._fore = style_item.GetFore()
@@ -339,6 +343,7 @@ class CssItem(object):
         self._size = style_item.GetSize()
         # List of additional style specs
         self._decor = self.ExtractDecorators()
+        self._decor.extend(style_item.GetModifierList())
 
     def __eq__(self, css2):
         """Defines the == operator for the CssItem class
@@ -347,10 +352,7 @@ class CssItem(object):
         @rtype: bool
 
         """
-        if self.__str__() == str(css2):
-            return True
-        else:
-            return False
+        return self.__str__() == str(css2)
 
     def __str__(self):
         """Outputs the css item as a formatted css block
@@ -481,7 +483,7 @@ class LaTeX(plugin.Plugin):
     """Creates a LaTeX document object from the contents of the
     supplied document reference.
     @todo: performance improvements and wordwrap in generated document
-    
+
     """
     plugin.Implements(GeneratorI)
     def __init__(self, plgmgr):
@@ -490,6 +492,8 @@ class LaTeX(plugin.Plugin):
 
         """
         plugin.Plugin.__init__(self, plgmgr)
+
+        # Attributes
         self._stc = None
         self._id = ed_glob.ID_TEX_GEN
         self._dstyle = StyleItem()
@@ -527,7 +531,7 @@ class LaTeX(plugin.Plugin):
 
         # Get Document start point info
         last_id = self._stc.GetStyleAt(parse_pos)
-        tmp = self.TransformText(self._stc.GetTextRange(parse_pos, 
+        tmp = self.TransformText(self._stc.GetTextRange(parse_pos,
                                                         parse_pos + 1))
         tag = self._stc.FindTagById(last_id)
         if tag != wx.EmptyString:
@@ -648,7 +652,7 @@ class LaTeX(plugin.Plugin):
         """Returns a comma separated rgb string representation
         of the input hex string. 1.0 = White, 0.0 = Black.
         @param hex_str: hex string to convert to latex rgb format
-        
+
         """
         r_hex = hex_str
         if r_hex[0] == u"#":
@@ -666,8 +670,8 @@ class LaTeX(plugin.Plugin):
         supplied StyleItem.
         @param cmd_name: name of command
         @param s_item: style item to create command for
-        @postcondition: new styling command is created and registered for use 
-        
+        @postcondition: new styling command is created and registered for use
+
         """
         cmd_name = self.CreateCmdName(cmd_name)
         if cmd_name in self._cmds:
@@ -711,7 +715,7 @@ class LaTeX(plugin.Plugin):
         escaping all special characters and sequences.
         @param txt: text to transform
         @return: txt with all special characters transformed
-        
+
         """
         ch_map = { "#" : "\\#", "$" : "\\$", "^" : "\\^",
                    "%" : "\\%", "&" : "\\&", "_" : "\\_",
@@ -728,10 +732,10 @@ class LaTeX(plugin.Plugin):
 #-----------------------------------------------------------------------------#
 
 class Rtf(plugin.Plugin):
-    """Generates a fully styled RTF document from the given text 
+    """Generates a fully styled RTF document from the given text
     controls contents.
     @todo: add support for bold/italic/underline and multiple fonts
-    
+
     """
     plugin.Implements(GeneratorI)
     def __init__(self, mgr):
@@ -741,6 +745,8 @@ class Rtf(plugin.Plugin):
 
         """
         plugin.Plugin.__init__(self, mgr)
+
+        # Attributes
         self._stc = None
         self._id = ed_glob.ID_RTF_GEN
         self._colortbl = RtfColorTbl()
@@ -813,7 +819,7 @@ class Rtf(plugin.Plugin):
         returning the RTF equvialent of the given stc_doc
         @param stc_doc: document to generate text from
         @return: document marked up in rtf
-    
+
         """
         self._stc = stc_doc
         return ('rtf', self._GenRtf())
@@ -832,7 +838,7 @@ class Rtf(plugin.Plugin):
         @return: menu entry item for this generator
 
         """
-        return wx.MenuItem(menu, self._id, _("Generate %s") % u"RTF", 
+        return wx.MenuItem(menu, self._id, _("Generate %s") % u"RTF",
                            _("Generate a %s version of the " \
                              "current document") % u"RTF")
 
@@ -849,6 +855,8 @@ class Rtf(plugin.Plugin):
             tmp = tmp + chmap.get(char, char)
         return tmp
 
+#-----------------------------------------------------------------------------#
+
 class RtfColorTbl(object):
     """A storage class to help with generating the color table for
     the Rtf Generator Class.
@@ -861,7 +869,7 @@ class RtfColorTbl(object):
 
         """
         object.__init__(self)
-        
+
         # Attributes
         self._index = list() # manages the order of the tables keys
         self._tbl = dict()   # map of style item color vals to rtf defs
