@@ -16,8 +16,8 @@ and setting of the program by setting values in the Profile.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: prefdlg.py 53301 2008-04-22 03:37:58Z CJP $"
-__revision__ = "$Revision: 53301 $"
+__svnid__ = "$Id: prefdlg.py 54780 2008-07-24 02:56:08Z CJP $"
+__revision__ = "$Revision: 54780 $"
 
 #----------------------------------------------------------------------------#
 # Dependancies
@@ -320,7 +320,7 @@ class GeneralPanel(PrefPanelBase):
         self.SetToolTipString(_("Changes made in this dialog are saved in your "
                                 "current profile. Some Items such as Language "
                                 "require the program to be restarted before "
-                                "taking affect."))
+                                "taking effect."))
         self.__DoLayout()
 
         # Event Handlers
@@ -879,7 +879,7 @@ class DocSyntaxPanel(wx.Panel):
             Profile_Set(ed_glob.ID_2_PROF[e_id], e_obj.GetValue())
 
             for mainw in wx.GetApp().GetMainWindows():
-                mainw.nb.UpdateTextControls()
+                mainw.nb.UpdateTextControls('UpdateAllStyles')
 
         elif e_id == ed_glob.ID_SYNTAX:
             val = e_obj.GetValue()
@@ -914,6 +914,7 @@ class AppearancePanel(PrefPanelBase):
         self.Bind(wx.EVT_CHOICE, self.OnChoice)
         self.Bind(wx.EVT_SLIDER, self.OnSetTransparent, \
                   id=ed_glob.ID_TRANSPARENCY)
+        self.Bind(ed_event.EVT_NOTIFY, self.OnFontChange)
 
     def _DoLayout(self):
         """Add and layout the widgets
@@ -935,6 +936,9 @@ class AppearancePanel(PrefPanelBase):
                            (ExChoice(self, ed_glob.ID_PREF_ICONSZ,
                                      ['16', '24', '32'],
                              str(Profile_Get('ICON_SZ', 'size_tuple')[0])), 0)])
+        tabicon_cb = wx.CheckBox(self, ed_glob.ID_PREF_TABICON,
+                                 _("Show Icons on Tabs"))
+        tabicon_cb.SetValue(Profile_Get('TABICONS'))
 
         # Layout Section
         mainw = wx.GetApp().GetActiveWindow()
@@ -954,34 +958,46 @@ class AppearancePanel(PrefPanelBase):
         wp_cb = wx.CheckBox(self, ed_glob.ID_PREF_WPOS, \
                             _("Remember Window Position on Exit"))
         wp_cb.SetValue(Profile_Get('SET_WPOS'))
+        sb_cb = wx.CheckBox(self, ed_glob.ID_SHOW_SB, _("Show Status Bar"))
+        sb_cb.SetValue(Profile_Get('STATBAR'))
+        tb_cb = wx.CheckBox(self, ed_glob.ID_VIEW_TOOL, _("Show Toolbar"))
+        tb_cb.SetValue(Profile_Get('TOOLBAR'))
+
+        # Font
+        fnt = Profile_Get('FONT3', 'font', wx.NORMAL_FONT)
+        fpick = PyFontPicker(self, wx.ID_ANY, fnt)
+        fpick.SetToolTipString(_("Main display font for various UI components"))
 
         # Misc
         trans_size = (-1, -1)
         if wx.Platform == '__WXGTK__':
             trans_size = (200, 15)
-        tsizer = wx.BoxSizer(wx.HORIZONTAL)
-        tsizer.AddMany([(wx.StaticText(self, label=_("Transparency") + u": "),
-                         0), ((5, 5), 0),
-                        (wx.Slider(self, ed_glob.ID_TRANSPARENCY,
-                                   Profile_Get('ALPHA'), 100, 255,
-                                   size=trans_size, style=wx.SL_HORIZONTAL | \
-                                   wx.SL_AUTOTICKS | wx.SL_LABELS), 0)])
 
         # Layout
-        sizer = wx.FlexGridSizer(10, 2, 5, 5)
+        sizer = wx.FlexGridSizer(16, 2, 5, 5)
         sizer.AddMany([((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("Icons") + u": "), 0,
                         wx.ALIGN_CENTER_VERTICAL), (iconsz, 0),
                        ((5, 5), 0), (tbiconsz, 0),
+                       ((5, 5), 0), (tabicon_cb, 0),
                        ((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("Layout") + u": "), 0,
                         wx.ALIGN_CENTER_VERTICAL),
                         (perspec_sz, 0, wx.ALIGN_CENTER_VERTICAL),
                        ((5, 5), 0), (ws_cb, 0),
                        ((5, 5), 0), (wp_cb, 0),
+                       ((5, 5), 0), (sb_cb, 0),
+                       ((5, 5), 0), (tb_cb, 0),
                        ((10, 10), 0), ((10, 10), 0),
-                       (wx.StaticText(self, label=_("Misc") + u": "), 0),
-                       (tsizer, 0),
+                       (wx.StaticText(self, label=_("Transparency") + u": "), 0),
+                       (wx.Slider(self, ed_glob.ID_TRANSPARENCY,
+                                   Profile_Get('ALPHA'), 100, 255,
+                                   size=trans_size, style=wx.SL_HORIZONTAL | \
+                                   wx.SL_AUTOTICKS | wx.SL_LABELS), 0, wx.EXPAND),
+                       ((10, 10), 0), ((10, 10), 0),
+                       (wx.StaticText(self, label=_("Display Font") + u": "),
+                        0, wx.ALIGN_CENTER_VERTICAL),
+                       (fpick, 1, wx.EXPAND),
                        ((10, 10), 0), ((10, 10), 0),
                        ])
 
@@ -996,9 +1012,23 @@ class AppearancePanel(PrefPanelBase):
 
         """
         e_id = evt.GetId()
-        if e_id in [ed_glob.ID_PREF_WPOS, ed_glob.ID_PREF_WSIZE]:
-            Profile_Set(ed_glob.ID_2_PROF[e_id],
-                        evt.GetEventObject().GetValue())
+        evalue = evt.GetEventObject().GetValue()
+        if e_id in (ed_glob.ID_PREF_WPOS, ed_glob.ID_PREF_WSIZE):
+            Profile_Set(ed_glob.ID_2_PROF[e_id], evalue)
+        elif e_id == ed_glob.ID_PREF_TABICON:
+            Profile_Set(ed_glob.ID_2_PROF[e_id], evalue)
+            ed_msg.PostMessage(ed_msg.EDMSG_THEME_NOTEBOOK)
+        elif e_id in (ed_glob.ID_SHOW_SB, ed_glob.ID_VIEW_TOOL):
+            Profile_Set(ed_glob.ID_2_PROF[e_id], evalue)
+            if e_id == ed_glob.ID_SHOW_SB:
+                fun = 'GetStatusBar'
+            else:
+                fun = 'GetToolBar'
+
+            # Update Window(s)
+            for mainw in wx.GetApp().GetMainWindows():
+                getattr(mainw, fun)().Show(evalue)
+                mainw.SendSizeEvent()
         else:
             evt.Skip()
 
@@ -1024,6 +1054,14 @@ class AppearancePanel(PrefPanelBase):
         else:
             evt.Skip()
 
+    @staticmethod
+    def OnFontChange(evt):
+        """Send out update messages fo rdisplay font changes"""
+        font = evt.GetValue()
+        if isinstance(font, wx.Font) and not font.IsNull():
+            Profile_Set('FONT3', font, 'font')
+            ed_msg.PostMessage(ed_msg.EDMSG_DSP_FONT, font)
+
     @staticmethod
     def OnSetTransparent(evt):
         """Sets the transparency of the editor while the slider
@@ -1353,6 +1391,8 @@ KEYS = ['', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
         'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
         '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '+', 'Left',
         'Right', 'Down', 'Up', 'Home', 'End', 'Tab']
+KEYS.extend(["F" + str(x) for x in range(1, 13)])
+
 if wx.Platform == '__WXMSW__':
     KEYS.remove('Tab')
 KEYS.sort()
@@ -1679,6 +1719,9 @@ class KeyBindingPanel(wx.Panel):
             if not len(csel):
                 csel = None
             self.binder.LoadKeyProfile(csel)
+            Profile_Set('KEY_PROFILE', csel)
+            ed_msg.PostMessage(ed_msg.EDMSG_MENU_REBIND)
+            self.menub.SaveKeyProfile()
             self.EnableControls(csel is not None)
         elif e_id == ID_MENUS:
             mi_listbx = self.FindWindowById(ID_MENU_ITEMS)
