@@ -20,8 +20,8 @@ happen during run time.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: profiler.py 53301 2008-04-22 03:37:58Z CJP $"
-__revision__ = "$Revision: 53301 $"
+__svnid__ = "$Id: profiler.py 55000 2008-08-07 00:03:20Z CJP $"
+__revision__ = "$Revision: 55000 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -60,6 +60,7 @@ _DEFAULTS = {
            'ICONS'      : 'Tango',          # Icon Theme
            'ICON_SZ'    : (24, 24),         # Toolbar Icon Size
            'INDENTWIDTH': 4,                # Default indent width
+           'ISBINARY'   : False,            # Is this instance a binary
            'KEY_PROFILE': None,             # Keybinding profile
            'LANG'       : 'Default',        # UI language
            'LASTCHECK'  : 0,                # Last time update check was done
@@ -72,24 +73,28 @@ _DEFAULTS = {
            'REPORTER'   : True,             # Error Reporter is Active
            'SAVE_POS'   : True,             # Remember Carat positions
            'SAVE_SESSION' : False,          # Load previous session on startup
+           'SESSION_KEY' : '',              # Ipc Session Server Key
            'SET_WPOS'   : True,             # Remember window position
            'SET_WSIZE'  : True,             # Remember mainwindow size on exit
            'SHOW_EDGE'  : True,             # Show Edge Guide
            'SHOW_EOL'   : False,            # Show EOL markers
            'SHOW_LN'    : True,             # Show Line Numbers
            'SHOW_WS'    : False,            # Show whitespace markers
+           'STATBAR'    : True,             # Show Status Bar
            'SYNTAX'     : True,             # Use Syntax Highlighting
            'SYNTHEME'   : 'Default',        # Syntax Highlight color scheme
+           'TABICONS'   : True,             # Show Tab Icons
            'TABWIDTH'   : 8,                # Tab width
            'THEME'      : 'DEFAULT',        # For future use
            'TOOLBAR'    : True,             # Show Toolbar
-           'USETABS'    : False,             # Use tabs instead of spaces
+           'USETABS'    : False,            # Use tabs instead of spaces
            'USE_PROXY'  : False,            # Use Proxy server settings?
            'VI_EMU'     : False,            # Use Vi emulation mode
            'WRAP'       : False,            # Use Wordwrap
            'WSIZE'      : (700, 450)        # Mainwindow size
           #FONT1 created at runtime by ed_styles as primary font
           #FONT2 created at runtime by ed_styles as secondary font
+          #FONT3 Standard Font by UI, added to profile when customized
 }
 
 #--------------------------------------------------------------------------#
