@@ -29,11 +29,13 @@ In addition to the L{SyntaxMgr} there are also a number of other utility and
 convienience functions in this module for accessing data from other related
 objects such as the Extension Register.
 
+@summary: Main api access point for the syntax package.
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: syntax.py 52351 2008-03-06 08:36:41Z CJP $"
-__revision__ = "$Revision: 52351 $"
+__svnid__ = "$Id: syntax.py 54546 2008-07-08 03:42:50Z CJP $"
+__revision__ = "$Revision: 54546 $"
 
 #-----------------------------------------------------------------------------#
 # Dependencies
@@ -56,6 +58,8 @@ SYNSPEC    = 2    # Highligter specs
 PROPERTIES = 3    # Extra Properties
 LANGUAGE   = 4    # Language ID
 COMMENT    = 5    # Gets the comment characters pattern
+CLEXER     = 6    # Container Lexer Styler Method
+INDENTER   = 7    # Auto-indenter method
 
 _ = wx.GetTranslation
 #-----------------------------------------------------------------------------#
@@ -168,10 +172,10 @@ class SyntaxMgr(object):
         return True
 
     def SyntaxData(self, ext):
-        """Fetches the language data based on a file extention string.
-        The file extension is used to look up the default lexer actions from the
-        EXT_REG dictionary.
-        @see L{synglob}
+        """Fetches the language data based on a file extention string. The file
+        extension is used to look up the default lexer actions from the EXT_REG
+        dictionary.
+        @see: L{synglob}
         @param ext: a string representing the file extension
         @return: Returns a Dictionary of Lexer Config Data
 
@@ -197,8 +201,13 @@ class SyntaxMgr(object):
         syn_data[PROPERTIES] = mod.Properties(lex_cfg[LANG_ID])
         syn_data[LANGUAGE] = lex_cfg[LANG_ID]
         syn_data[COMMENT] = mod.CommentPattern(lex_cfg[LANG_ID])
-        return syn_data
+        if syn_data[LEXER] == wx.stc.STC_LEX_CONTAINER:
+            syn_data[CLEXER] = mod.StyleText
+        else:
+            syn_data[CLEXER] = None
+        syn_data[INDENTER] = getattr(mod, 'AutoIndenter', None)
 
+        return syn_data
 
 #-----------------------------------------------------------------------------#
 
@@ -281,7 +290,6 @@ def GetExtFromId(ext_id):
     """
     extreg = ExtensionRegister()
     ftype = synglob.GetDescriptionFromId(ext_id)
-    print "HELLO", ftype
     return extreg[ftype][0]
 
 def GetIdFromExt(ext):
